@@ -60,10 +60,10 @@ device_config (void)
 }
 
 sw_handle *
-device_open (int flags)
+device_open (int cueing, int flags)
 {
   if (pref->open)
-    return pref->open (flags);
+    return pref->open (cueing, flags);
   else
     return NULL;
 }
