@@ -154,7 +154,9 @@ sweep_dir_exists (char * pathname)
       info_dialog_new (_("Directory does not exist"), NULL, buf);
       g_free (dirname);
       return FALSE;
+      break;
     default:
+      break;
     }
   }
 
@@ -163,6 +165,22 @@ sweep_dir_exists (char * pathname)
   return TRUE;
 }
 
+static gboolean
+file_guess_mp3 (char * pathname)
+{
+  gchar * ext;
+
+  ext = strrchr (pathname, '.');
+
+  if (ext != NULL) {
+    ext++;
+    if (!g_strncasecmp (ext, "mp3", 4)) return TRUE;
+    if (!g_strncasecmp (ext, "mp2", 4)) return TRUE;
+  }
+
+  return FALSE;
+}
+
 sw_sample *
 sample_reload (sw_sample * sample)
 {
@@ -181,7 +199,7 @@ sample_reload (sw_sample * sample)
 #endif
 
 #ifdef HAVE_MAD
-  if (new_sample == NULL)
+  if (new_sample == NULL && file_guess_mp3 (sample->pathname))
     new_sample = mad_sample_reload (sample);
 #endif
 
@@ -209,7 +227,7 @@ try_sample_load (char * pathname)
 #endif
 
 #ifdef HAVE_MAD
-  if (sample == NULL)
+  if (sample == NULL && file_guess_mp3 (pathname))
     sample = mad_sample_load (pathname);
 #endif
 
@@ -228,7 +246,8 @@ sample_load (char * pathname)
     } else if (access(pathname, R_OK) == -1) {
       switch (errno) {
       case ENOENT:
-	sweep_perror (errno, pathname);
+	create_sample_new_dialog_defaults (pathname);
+	/*sweep_perror (errno, pathname);*/
 	break;
       default:
 	sweep_perror (errno, _("Unable to read\n%s"), pathname);
@@ -247,7 +266,7 @@ sample_load (char * pathname)
 static void
 sample_load_destroy_cb (GtkWidget * widget, gpointer data)
 {
-  sample_bank_remove (NULL);
+  /*sample_bank_remove (NULL);*/
 }
 
 static void
@@ -741,10 +760,6 @@ sample_save_as_cb(GtkWidget * widget, gpointer data)
   gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_CENTER);
   gtk_widget_set_usize (filesel, win_width, win_height);
 
-  printf ("about to save %s\n", sample->pathname);
-  fflush (stdout);
-  sleep (1);
-
   if (!strcmp (g_dirname (sample->pathname), ".")) {
 
     last_save = prefs_get_string (LAST_SAVE_KEY);
