@@ -620,7 +620,7 @@ create_sndfile_encoding_options_dialog (sndfile_save_options * so)
 
 #if 1
   if (!so->saving) {
-    entry = samplerate_chooser_new ();
+    entry = samplerate_chooser_new (NULL);
     gtk_table_attach (GTK_TABLE(table), entry, 0, 2, 1, 2,
 		      GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
     gtk_widget_show (entry);
@@ -693,7 +693,7 @@ create_sndfile_encoding_options_dialog (sndfile_save_options * so)
   /* Channels */
 
 #if 1
-  entry = channelcount_chooser_new ();
+  entry = channelcount_chooser_new (NULL);
   gtk_table_attach (GTK_TABLE(table), entry, 0, 2, 2, 3,
 		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
   gtk_widget_show (entry);
