@@ -80,7 +80,7 @@ sweep_sndfile_perror (SNDFILE * sndfile, char * pathname)
   char buf[BUF_LEN];
 
   sf_error_str (sndfile, buf, BUF_LEN);
-  
+
   sweep_perror (errno, "libsndfile: %s\n\n%s", buf,
 		(pathname == NULL) ? "" : pathname);
 }
@@ -677,9 +677,12 @@ _sndfile_sample_load (sw_sample * sample, char * pathname, SF_INFO * sfinfo,
 
   sf_data * sf;
 
-#define RAW_ERR_STR \
+#define RAW_ERR_STR_1 \
 "Bad format specified for file open."
 
+#define RAW_ERR_STR_2 \
+"File opened for read. Format not recognised."
+
   if (sfinfo == NULL)
     sfinfo = g_malloc0 (sizeof (SF_INFO));
 
@@ -691,7 +694,8 @@ _sndfile_sample_load (sw_sample * sample, char * pathname, SF_INFO * sfinfo,
     }
 
     sf_error_str (NULL, buf, BUF_LEN);
-    if (!strncmp (buf, RAW_ERR_STR, BUF_LEN)) {
+    if (!strncmp (buf, RAW_ERR_STR_1, BUF_LEN) ||
+	!strncmp (buf, RAW_ERR_STR_2, BUF_LEN)) {
 
       so = g_malloc0 (sizeof(*so));
       so->saving = FALSE;
@@ -720,6 +724,8 @@ _sndfile_sample_load (sw_sample * sample, char * pathname, SF_INFO * sfinfo,
 	info_dialog_new (buf, NULL, message);
 	g_free (message);
       } else {
+	/* We've already got the error string so no need to call
+	 * sweep_sndfile_perror() here */
 	sweep_perror (errno, "libsndfile: %s\n\n%s", buf, pathname);
       }
 
