@@ -70,6 +70,8 @@
 #include <ogg/ogg.h>
 #include <speex.h>
 #include <speex_header.h>
+#include <speex_stereo.h>
+#include <speex_callbacks.h>
 
 #define BUFFER_LEN 1024
 
@@ -95,18 +97,52 @@
 
 #include "../pixmaps/xifish.xpm"
 
+/* MAINTENANCE:
+ *
+ * Upon release of Speex 1.0, force a requirement on that version in
+ * configure. Then, remove all references to SPEEX_HAVE_BETA4 (assume
+ * this is true, as those features will be available), and also assume
+ * that SPEEX_NB_MODES > 2. This should reduce the random ifdef'ing
+ * present to accomodate the flux of prerelease versions of Speex.
+ */
+
+#ifdef SPEEX_SET_DTX
+#define HAVE_SPEEX_BETA4
+#endif
+
 #define BUF_LEN 128
 
+#define MODE_KEY "Speex_Mode"
+#define FEATURES_KEY "Speex_Features"
 #define QUALITY_KEY "Speex_Quality"
-#define VBR_KEY "Speex_VBR"
+#define BR_KEY "Speex_BR"
+#define BITRATE_KEY "Speex_Bitrate"
 #define COMPLEXITY_KEY "Speex_Complexity"
 #define SERIALNO_KEY "OggSpeex_Serialno"
 #define FRAMEPACK_KEY "OggSpeex_FramePack"
 
-#define DEFAULT_QUALITY 3.0
+/* Mode choices */
+#define MODE_NARROWBAND 0
+#define MODE_WIDEBAND 1
+#define MODE_ULTRAWIDEBAND 2
+
+/* Feature flags */
+#define FEAT_VBR 1
+#define FEAT_VAD 2
+#define FEAT_DTX 4
+
+#ifdef HAVE_SPEEX_BETA4
+#define DEFAULT_FEATURES (FEAT_VBR | FEAT_VAD | FEAT_DTX)
+#else
+#define DEFAULT_FEATURES (FEAT_VBR)
+#endif
+
+#define DEFAULT_QUALITY 8.0
 #define DEFAULT_COMPLEXITY 3.0
 #define DEFAULT_FRAMEPACK 1
 
+#define DEFAULT_ENH_ENABLED 1
+
 extern GtkStyle * style_bw;
 
 #define READ_SIZE 200
@@ -174,23 +210,25 @@ file_is_ogg_speex (const char * pathname)
 }
 
 static void *
-process_header(ogg_packet *op, int enh_enabled, int *frame_size, int *rate,
-	       int *nframes, int forceMode)
+process_header(ogg_packet *op, int enh_enabled, int * frame_size, int * rate,
+	       int * nframes, int forceMode, int * channels,
+	       SpeexStereoState * stereo, int * extra_headers)
 {
   void *st;
   SpeexMode *mode;
   SpeexHeader *header;
   int modeID;
+  SpeexCallback callback;
 
   header = speex_packet_to_header((char*)op->packet, op->bytes);
   if (!header) {
-    fprintf (stderr, "Cannot read header\n");
+    info_dialog_new ("Speex error", NULL, "Speex: cannot read header");
     return NULL;
   }
   if (header->mode >= SPEEX_NB_MODES) {
-    fprintf (stderr,
-	     "Mode number %d does not (any longer) exist in this version\n",
-	     header->mode);
+    info_dialog_new ("Speex error", NULL,
+		     "Mode number %d does not (any longer) exist in this version\n",
+		     header->mode);
     return NULL;
   }
 
@@ -199,44 +237,86 @@ process_header(ogg_packet *op, int enh_enabled, int *frame_size, int *rate,
     modeID = forceMode;
   mode = speex_mode_list[modeID];
 
+#ifdef HAVE_SPEEX_BETA4
+  if (header->speex_version_id > 1) {
+    info_dialog_new ("Speex error", NULL,
+		     "This file was encoded with Speex bit-stream version %d, "
+		     "which I don't know how to decode\n",
+		     header->speex_version_id);
+    return NULL;
+  }
+#endif
+
   if (mode->bitstream_version < header->mode_bitstream_version) {
-    fprintf (stderr,
-	     "The file was encoded with a newer version of Speex. "
-	     "You need to upgrade in order to play it.\n");
+    info_dialog_new ("Speex error", NULL,
+		     "The file was encoded with a newer version of Speex. "
+		     "You need to upgrade in order to play it.\n");
     return NULL;
   }
 
   if (mode->bitstream_version > header->mode_bitstream_version) {
-    fprintf (stderr,
-	     "The file was encoded with an older version of Speex. "
-	     "You would need to downgrade the version in order to play it.\n");
+    info_dialog_new ("Speex error", NULL,
+		     "The file was encoded with an older version of Speex. "
+		     "You would need to downgrade the version in order to play it.\n");
     return NULL;
   }
 
   st = speex_decoder_init(mode);
+  if (!st) {
+    info_dialog_new ("Speex error", NULL,
+		     "Decoder initialization failed.\n");
+    return NULL;
+  }
+
   speex_decoder_ctl(st, SPEEX_SET_ENH, &enh_enabled);
   speex_decoder_ctl(st, SPEEX_GET_FRAME_SIZE, frame_size);
-  
-  /* FIXME: need to adjust in case the forceMode option is set */
-  *rate = header->rate;
-  if (header->mode==1 && forceMode==0)
-    *rate/=2;
-  if (header->mode==0 && forceMode==1)
-    *rate*=2;
+
+  if (!(*channels==1))
+    {
+      callback.callback_id = SPEEX_INBAND_STEREO;
+      callback.func = speex_std_stereo_request_handler;
+      callback.data = stereo;
+      speex_decoder_ctl(st, SPEEX_SET_HANDLER, &callback);
+    }
+  if (!*rate)
+    *rate = header->rate;
+  /* Adjust rate if --force-* options are used */
+  if (forceMode!=-1)
+    {
+      if (header->mode < forceMode)
+	*rate <<= (forceMode - header->mode);
+      if (header->mode > forceMode)
+	*rate >>= (header->mode - forceMode);
+    }
+
+  speex_decoder_ctl(st, SPEEX_SET_SAMPLING_RATE, rate);
+
   *nframes = header->frames_per_packet;
 
+  if (*channels == -1)
+    *channels = header->nb_channels;
+
 #ifdef DEBUG  
   fprintf (stderr, "Decoding %d Hz audio using %s mode",
 	   *rate, mode->modeName);
-  
+
+  if (*channels==1)
+      fprintf (stderr, " (mono");
+   else
+      fprintf (stderr, " (stereo");
+
   if (header->vbr)
     fprintf (stderr, " (VBR)\n");
   else
     fprintf(stderr, "\n");
-  /*fprintf (stderr, "Decoding %d Hz audio at %d bps using %s mode\n",
-   *rate, mode->bitrate, mode->modeName);*/
 #endif
-  
+
+#ifdef HAVE_SPEEX_BETA4
+  *extra_headers = header->extra_headers;
+#else
+  *extra_headers = 0;
+#endif
+
   free(header);
 
   return st;
@@ -254,6 +334,9 @@ sample_load_speex_data (sw_op_instance * inst)
   SpeexBits bits;
   int frame_size = 0;
   int rate;
+  int channels = -1;
+  int extra_headers;
+  SpeexStereoState stereo = SPEEX_STEREO_STATE_INIT;
   int packet_count = 0;
   int stream_init = 0;
 
@@ -264,7 +347,7 @@ sample_load_speex_data (sw_op_instance * inst)
 
   char * ogg_data;
 
-  int enh_enabled = 0;
+  int enh_enabled = DEFAULT_ENH_ENABLED;
   int nframes = 2;
   int eos = 0;
   int forceMode = -1;
@@ -280,12 +363,12 @@ sample_load_speex_data (sw_op_instance * inst)
 
   fd = open (sample->pathname, O_RDONLY);
   if (fd == -1) {
-    perror ("bugger\n");
+    sweep_perror (errno, "failed open in sample_load_speex_data");
     return NULL;
   }
 
   if (fstat (fd, &statbuf) == -1) {
-    perror ("statty stat\n");
+    sweep_perror (errno, "failed stat in sample_load_speex_data");
     return NULL;
   }
 
@@ -307,7 +390,7 @@ sample_load_speex_data (sw_op_instance * inst)
       ogg_data = ogg_sync_buffer (&oy, n);
       nread = read (fd, ogg_data, n);
       if (nread == -1) {
-	perror (NULL);
+	sweep_perror (errno, "speex: %s", sample->pathname);
 	active = FALSE;
       } else if (nread == 0) {
 	/* eof */
@@ -329,55 +412,60 @@ sample_load_speex_data (sw_op_instance * inst)
 
 	/* Extract all available packets */
 	while (!eos && ogg_stream_packetout (&os, &op) == 1) {
-	  switch (packet_count) {
-
-	  case 0: /* header */
+	  if (packet_count == 0) {/* header */
 	    st = process_header (&op, enh_enabled, &frame_size, &rate,
-				 &nframes, forceMode);
+				 &nframes, forceMode, &channels, &stereo,
+				 &extra_headers);
 	    if (st == NULL) {
-	      printf ("Not Speex!\n");
-	      return NULL;
+	      /*printf ("Not Speex!\n");*/
+	      active = FALSE;
 	    }
 
 	    sample->sounddata->format->rate = rate;
+	    sample->sounddata->format->channels = channels;
 
 	    if (nframes == 0)
 	      nframes = 1;
 
-
-	    break;
-
-	  case 1: /* XXX: metadata */
-	    break;
-
-	  default:
+	  } else if (packet_count <= 1+extra_headers) {
+	    /* XXX: metadata, extra_headers: ignore */
+	  } else {
 	    if (op.e_o_s)
 	      eos = 1;
 
 	    /* Copy Ogg packet to Speex bitstream */
 	    speex_bits_read_from (&bits, (char *)op.packet, op.bytes);
 
-
 	    frames_total += nframes * frame_size;
 
 	    if (sample->sounddata->nr_frames != frames_total) {
 	      sample->sounddata->data =
 		g_realloc (sample->sounddata->data,
-			   frames_total * sizeof (sw_audio_t));
+			   frames_total * channels * sizeof (sw_audio_t));
 	    }
 
 	    sample->sounddata->nr_frames = frames_total;
 
-	    d = &((sw_audio_t *)sample->sounddata->data)[frames_decoded];
+	    d = &((sw_audio_t *)sample->sounddata->data)
+		  [frames_decoded * channels];
 
 	    if (d != NULL) {
 	      for (j = 0; j < nframes; j++) {
 		/* Decode frame */
 		speex_decode (st, &bits, d);
-		for (i = 0; i < frame_size; i++) {
-		  d[i] /= 32767;
+#ifdef DEBUG
+		if (speex_bits_remaining (&bits) < 0) {
+		  info_dialog_new ("Speex warning", NULL,
+				   "Speex: decoding overflow -- corrupted stream at frame %ld", frames_decoded + (j * frame_size));
 		}
-		d += frame_size;
+#endif
+		if (channels == 2)
+		  speex_decode_stereo (d, frame_size, &stereo);
+
+		for (i = 0; i < frame_size * channels; i++) {
+		  d[i] /= 32767.0;
+		}
+		d += (frame_size * channels);
 		frames_decoded += frame_size;
 	      }
 	    }
@@ -403,10 +491,12 @@ sample_load_speex_data (sw_op_instance * inst)
 
   close (fd);
 
-  stat (sample->pathname, &statbuf);
-  sample->last_mtime = statbuf.st_mtime;
-  sample->edit_ignore_mtime = FALSE;
-  sample->modified = FALSE;
+  if (remaining <= 0) {
+    stat (sample->pathname, &statbuf);
+    sample->last_mtime = statbuf.st_mtime;
+    sample->edit_ignore_mtime = FALSE;
+    sample->modified = FALSE;
+  }
 
   sample_set_edit_state (sample, SWEEP_EDIT_STATE_DONE);
 
@@ -573,8 +663,11 @@ void comment_add(char **comments, int* length, char *tag, char *val)
 
 typedef struct {
   gchar * pathname;
-  gboolean use_vbr;
-  gint quality;
+  gint mode;
+  gint features;
+  gboolean use_br;
+  gfloat quality; /* either way */
+  gint bitrate;
   gint complexity;
   gint framepack;
   long serialno;
@@ -619,7 +712,7 @@ speex_sample_save_thread (sw_op_instance * inst)
   int comments_length = 0;
 
   int eos = 0;
-  int i, j, k;
+  int i, j;
 
   gboolean active = TRUE;
 
@@ -642,11 +735,6 @@ speex_sample_save_thread (sw_op_instance * inst)
   remaining = nr_frames;
   run_total = 0;
 
-  if (format->rate != 8000 && format->rate != 16000) {
-    fprintf (stderr, "Unsupported rate %d Hz for Speex encoding\n", format->rate);
-    return -1;
-  }
-
 #if 0
   if (format->channels != 1) {
     fprintf (stderr, "Unsupported channel count for Speex encoding\n");
@@ -659,17 +747,27 @@ speex_sample_save_thread (sw_op_instance * inst)
     return -1;
   }
 
-  if (format->rate == 8000) {
+  switch (so->mode) {
+  case MODE_NARROWBAND:
     mode = &speex_nb_mode;
-  } else if (format->rate == 16000) {
+    break;
+  case MODE_WIDEBAND:
     mode = &speex_wb_mode;
-  } else {
-    g_assert_not_reached ();
+    break;
+#if (SPEEX_NB_MODES > 2)
+  case MODE_ULTRAWIDEBAND:
+    mode = &speex_uwb_mode;
+    break;
+#endif
+  default:
+    mode = &speex_nb_mode;
+    break;
   }
 
-  speex_init_header (&header, format->rate, format->channels, mode);
+  speex_init_header (&header, format->rate, 1 , mode);
   header.frames_per_packet = so->framepack;
-  header.vbr = so->use_vbr;
+  header.vbr = (so->features & FEAT_VBR) ? 1 : 0;
+  header.nb_channels = format->channels;
 
 #ifdef DEBUG
   fprintf (stderr, "Encoding %d Hz audio using %s mode\n",
@@ -726,15 +824,37 @@ speex_sample_save_thread (sw_op_instance * inst)
 
   if (comments) g_free (comments);
 
+  speex_encoder_ctl (st, SPEEX_SET_SAMPLING_RATE, &format->rate);
+
   speex_encoder_ctl (st, SPEEX_GET_FRAME_SIZE, &frame_size);
   speex_encoder_ctl (st, SPEEX_SET_COMPLEXITY, &so->complexity);
-  speex_encoder_ctl (st, SPEEX_SET_QUALITY, &so->quality);
-  if (so->use_vbr) {
+  if (so->features & FEAT_VBR) {
     int tmp = 1;
     speex_encoder_ctl (st, SPEEX_SET_VBR, &tmp);
     speex_encoder_ctl (st, SPEEX_SET_VBR_QUALITY, &so->quality);
+#ifdef HAVE_SPEEX_BETA4
+    if (so->use_br) {
+      speex_encoder_ctl (st, SPEEX_SET_ABR, &so->bitrate);
+    }
+#endif
+  } else {
+    int tmp = (int)floor(so->quality);
+    speex_encoder_ctl (st, SPEEX_SET_QUALITY, &tmp);
+    if (so->use_br) {
+      speex_encoder_ctl (st, SPEEX_SET_BITRATE, &so->bitrate);
+    }
   }
 
+#ifdef HAVE_SPEEX_BETA4
+  if (so->features & FEAT_VAD) {
+    int tmp = 1;
+    speex_encoder_ctl (st, SPEEX_SET_VAD, &tmp);
+    if (so->features & FEAT_DTX) {
+      speex_encoder_ctl (st, SPEEX_SET_DTX, &tmp);
+    }
+  }
+#endif
+
   speex_bits_init (&bits);
 
   while (!eos) {
@@ -759,17 +879,16 @@ speex_sample_save_thread (sw_op_instance * inst)
 	  
 	  d = &((sw_audio_t *)sample->sounddata->data)
 	    [run_total * format->channels];
-
-	  memset (input, 0, sizeof(input));
+	  
+	  memcpy (input, d, sizeof (sw_audio_t) * len * format->channels);
 	  
 	  /* rip channel 0 out, in required format */
-	  for (j = 0; j < len; j++) {
-	    for (k = 0; k < format->channels; k++) {
-	      input[j] += *d++;
-	    }
+	  for (j = 0; j < len * format->channels; j++) {
 	    input[j] *= 32767.0;
 	  }
-	  
+
+	  if (format->channels == 2)
+	    speex_encode_stereo (input, len, &bits);
 	  speex_encode (st, input, &bits);
 	  
 	  remaining -= len;
@@ -778,7 +897,8 @@ speex_sample_save_thread (sw_op_instance * inst)
 	  percent = run_total / cframes;
 	  sample_set_progress_percent (sample, percent);
 	} else {
-	  speex_bits_pack (&bits, 0, 7);
+	  /*speex_bits_pack (&bits, 0, 7);*/
+	  speex_bits_pack (&bits, 15, 5);
 	}
 
 	id++;
@@ -956,9 +1076,9 @@ speex_save_options_dialog_ok_cb (GtkWidget * widget, gpointer data)
   GtkWidget * entry;
   gchar * text;
 
-  gboolean use_vbr;
+  gboolean use_br;
   GtkObject * adj; 
-  int quality, complexity, framepack;
+  int mode, features, quality, bitrate, complexity, framepack;
   gboolean rem_encode;
   long serialno;
   gboolean rem_serialno;
@@ -969,10 +1089,16 @@ speex_save_options_dialog_ok_cb (GtkWidget * widget, gpointer data)
 
   dialog = gtk_widget_get_toplevel (widget);
 
-  checkbutton =
-    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "vbr_chb"));
-  use_vbr =
-    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton));  
+  /* Mode */
+
+  mode =
+    GPOINTER_TO_INT(gtk_object_get_data (GTK_OBJECT(dialog), "mode_choice"));
+
+  /* Features */
+
+  features =
+    GPOINTER_TO_INT(gtk_object_get_data (GTK_OBJECT(dialog),
+					 "features_choice"));
 
   adj = GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "quality_adj"));
   quality = (int)GTK_ADJUSTMENT(adj)->value;
@@ -982,7 +1108,17 @@ speex_save_options_dialog_ok_cb (GtkWidget * widget, gpointer data)
 
   adj = GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "framepack_adj"));
   framepack = (int)GTK_ADJUSTMENT(adj)->value;
-  
+
+  checkbutton =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_chb"));
+  use_br = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton));
+
+  entry =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_entry"));
+  text = gtk_entry_get_text (GTK_ENTRY(entry));
+  bitrate = (int)strtol (text, (char **)NULL, 0);
+
+  /* rem encode */
   checkbutton =
     GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "rem_encode_chb"));
   rem_encode =
@@ -1004,8 +1140,11 @@ speex_save_options_dialog_ok_cb (GtkWidget * widget, gpointer data)
   gtk_widget_destroy (dialog);
 
   if (rem_encode) {
-    prefs_set_int (VBR_KEY, use_vbr);
+    prefs_set_int (MODE_KEY, mode);
+    prefs_set_int (FEATURES_KEY, features);
+    prefs_set_int (BR_KEY, use_br);
     prefs_set_int (QUALITY_KEY, quality);
+    prefs_set_int (BITRATE_KEY, bitrate);
     prefs_set_int (COMPLEXITY_KEY, complexity);
     prefs_set_int (FRAMEPACK_KEY, framepack);
   }
@@ -1020,9 +1159,11 @@ speex_save_options_dialog_ok_cb (GtkWidget * widget, gpointer data)
     g_free (sample->file_info);
   }
 
-  so->use_vbr = use_vbr;
-
+  so->mode = mode;
+  so->features = features;
+  so->use_br = use_br;
   so->quality = quality;
+  so->bitrate = bitrate;
   so->complexity = complexity;
   so->framepack = framepack;
 
@@ -1045,18 +1186,203 @@ speex_save_options_dialog_cancel_cb (GtkWidget * widget, gpointer data)
   sample_bank_remove (NULL);
 }
 
+typedef struct {
+  int number;
+  char * name;
+  char * desc;
+} sw_choice;
+
+static sw_choice mode_choices[] = {
+  { MODE_NARROWBAND, N_("Narrowband ~8 kHz (telephone quality)"), NULL },
+  { MODE_WIDEBAND, N_("Wideband ~16 kHz"), NULL },
+#if SPEEX_NB_MODES > 2
+  { MODE_ULTRAWIDEBAND, N_("Ultra-wideband 32-48 kHz"), NULL },
+#endif
+  { 0, NULL, NULL }
+};
+
+static sw_choice feature_choices[] = {
+#ifdef HAVE_SPEEX_BETA4
+  { 0, N_("Constant bitrate (CBR) with no features"),
+    NULL
+  },
+  { FEAT_VAD, N_("CBR with Voice Activity Detection (VAD)"),
+    N_("VAD generates low bitrate comfort noise to replace non-speech")
+  },
+  { FEAT_VAD | FEAT_DTX,
+    N_("CBR with VAD and Discontinuous Transmission (DTX)"),
+    N_("DTX marks extended pauses with a minimum bitrate signal")
+  },
+  { FEAT_VBR | FEAT_VAD | FEAT_DTX,
+    N_("Variable bitrate (VBR) with all features"),
+    N_("VBR allows the bitrate to adapt to the complexity of the speech, "
+       "and handles pauses using VAD and DTX")
+  },
+#else
+  { 0, N_("Constant bitrate (CBR)"), NULL },
+  { FEAT_VBR,
+    N_("Variable bitrate (VBR)"),
+    N_("VBR allows the bitrate to adapt to the complexity of the speech.")
+  },
+#endif
+  { 0, NULL, NULL }
+};
+
+static void
+speex_encode_options_update_cb (GtkWidget * widget, gpointer data)
+{
+  GtkWidget * dialog;
+  GtkWidget * br_checkbutton;
+  GtkWidget * quality_label;
+  GtkWidget * quality_hscale;
+  GtkWidget * br_label;
+  GtkWidget * br_entry;
+  GtkWidget * br_units;
+
+  gboolean br;
+  gint features;
+
+  dialog = gtk_widget_get_toplevel (widget);
+
+  /* Features */
+  features =
+    GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT(dialog),
+					  "features_choice"));
+
+  /* Quality */
+
+  quality_label =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "quality_label"));
+  quality_hscale =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "quality_hscale"));
+
+  /* Bitrate */
+
+  br_checkbutton =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_chb"));
+  br = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(br_checkbutton));
+
+  br_label =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_label"));
+  br_entry =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_entry"));
+  br_units =
+    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "br_units"));
+
+  gtk_widget_set_sensitive (br_label, br);
+  gtk_widget_set_sensitive (br_entry, br);
+  gtk_widget_set_sensitive (br_units, br);
+  gtk_widget_set_sensitive (quality_label, !br);
+  gtk_widget_set_sensitive (quality_hscale, !br);
+
+  if (features & FEAT_VBR) {
+    gtk_scale_set_digits (GTK_SCALE (quality_hscale), 1);
+    gtk_label_set_text (GTK_LABEL(br_label), _("Average bitrate"));
+  } else {
+    gtk_scale_set_digits (GTK_SCALE (quality_hscale), 0);
+    gtk_label_set_text (GTK_LABEL(br_label), _("Maximum bitrate"));
+  }
+
+}
+
+static void
+speex_encode_options_mode_cb (GtkWidget * widget, gpointer data)
+{
+  GtkWidget * dialog = GTK_WIDGET (data);
+  int mode;
+
+  mode = GPOINTER_TO_INT(gtk_object_get_user_data (GTK_OBJECT(widget)));
+
+  gtk_object_set_data (GTK_OBJECT(dialog), "mode_choice",
+		       GINT_TO_POINTER(mode));
+}
+
+static void
+speex_encode_options_mode_auto_cb (GtkWidget * widget, gpointer data)
+{
+  sw_sample * sample = (sw_sample *)data;
+  sw_format * f = sample->sounddata->format;
+  GtkWidget * dialog;
+  GtkOptionMenu * option_menu;
+  int mode;
+
+  dialog = gtk_widget_get_toplevel (widget);
+  option_menu =
+    GTK_OPTION_MENU(gtk_object_get_data (GTK_OBJECT(dialog), "mode_menu"));
+
+#if (SPEEX_NB_MODES > 2)
+  if (f->rate >= 32000) {
+    mode = MODE_ULTRAWIDEBAND;
+  } else
+#endif
+    if (f->rate >= 16000) {
+      mode = MODE_WIDEBAND;
+    } else {
+      mode = MODE_NARROWBAND;
+    }
+
+  gtk_option_menu_set_history (option_menu, mode);
+  gtk_object_set_data (GTK_OBJECT(dialog), "mode_choice",
+		       GINT_TO_POINTER(mode));
+}
+
+static void
+speex_encode_options_set_features (GtkWidget * dialog, int features)
+{
+  GtkOptionMenu * option_menu;
+  int i;
+
+  option_menu =
+    GTK_OPTION_MENU(gtk_object_get_data (GTK_OBJECT(dialog), "features_menu"));
+
+  for (i = 0; feature_choices[i].name != NULL; i++) {
+    if (feature_choices[i].number == features)
+      gtk_option_menu_set_history (option_menu, i);
+  }
+
+  gtk_object_set_data (GTK_OBJECT(dialog), "features_choice",
+		       GINT_TO_POINTER(features));
+}
+
+static void
+speex_encode_options_features_cb (GtkWidget * widget, gpointer data)
+{
+  GtkWidget * dialog = GTK_WIDGET (data);
+  int features;
+
+  features = GPOINTER_TO_INT(gtk_object_get_user_data (GTK_OBJECT(widget)));
+
+  gtk_object_set_data (GTK_OBJECT(dialog), "features_choice",
+		       GINT_TO_POINTER(features));
+
+  speex_encode_options_update_cb (dialog, data);
+}
+
+
 static void
 speex_encode_options_reset_cb (GtkWidget * widget, gpointer data)
 {
   GtkWidget * dialog;
-  GtkWidget * checkbutton;
 
-  gboolean use_vbr;
   GtkObject * adj;
-  int * i, quality, complexity, framepack;
+  int * i, features, quality, complexity, framepack;
 
   dialog = gtk_widget_get_toplevel (widget);
 
+  /* Mode menu */
+  speex_encode_options_mode_auto_cb (widget, data);
+
+  /* Features menu */
+  i = prefs_get_int (FEATURES_KEY);
+
+  if (i == NULL) {
+    features = DEFAULT_FEATURES;
+  } else {
+    features = *i;
+  }
+
+  speex_encode_options_set_features (dialog, features);
+
   /* Quality */
 
   adj = GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "quality_adj"));
@@ -1071,19 +1397,6 @@ speex_encode_options_reset_cb (GtkWidget * widget, gpointer data)
   
   gtk_adjustment_set_value (GTK_ADJUSTMENT(adj), (float)quality);
 
-  /* Use VBR */
-
-  i = prefs_get_int (VBR_KEY);
-  if (i == NULL) {
-    use_vbr = FALSE;
-  } else {
-    use_vbr = (gboolean) *i;
-  }
-
-  checkbutton =
-    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "vbr_chb"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbutton), use_vbr);
-
   /* Complexity */
 
   adj = GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "complexity_adj"));
@@ -1112,47 +1425,45 @@ speex_encode_options_reset_cb (GtkWidget * widget, gpointer data)
   
   gtk_adjustment_set_value (GTK_ADJUSTMENT(adj), (float)framepack);
 
+  speex_encode_options_update_cb (widget, data);
 }
 
 static void
 speex_encode_options_default_cb (GtkWidget * widget, gpointer data)
 {
   GtkWidget * dialog;
-  GtkWidget * checkbutton;
-
   GtkObject * quality_adj; 
   GtkObject * complexity_adj; 
   GtkObject * framepack_adj; 
 
   dialog = gtk_widget_get_toplevel (widget);
 
-  /* Quality */
+  /* Mode menu */
+  speex_encode_options_mode_auto_cb (widget, data);
 
+  /* Features menu */
+  speex_encode_options_set_features (dialog, DEFAULT_FEATURES);
+
+  /* Quality */
   quality_adj =
     GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "quality_adj"));
   gtk_adjustment_set_value (GTK_ADJUSTMENT(quality_adj), DEFAULT_QUALITY);
 
-  /* Use VBR */
-
-  checkbutton =
-    GTK_WIDGET(gtk_object_get_data (GTK_OBJECT(dialog), "vbr_chb"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbutton), FALSE);
-
   /* Complexity */
-
   complexity_adj =
     GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "complexity_adj"));
   gtk_adjustment_set_value (GTK_ADJUSTMENT(complexity_adj),
 			    DEFAULT_COMPLEXITY);
 
   /* Framepack */
-
   framepack_adj =
     GTK_OBJECT(gtk_object_get_data (GTK_OBJECT(dialog), "framepack_adj"));
   gtk_adjustment_set_value (GTK_ADJUSTMENT(framepack_adj), DEFAULT_FRAMEPACK);
 
+  speex_encode_options_update_cb (widget, data);
 }
 
+
 static void
 remember_serialno_clicked_cb (GtkWidget * widget, gpointer data)
 {
@@ -1207,6 +1518,8 @@ randomise_serialno_released_cb (GtkWidget * widget, gpointer data)
   gtk_timeout_remove (tag);
 }
 
+
+
 static GtkWidget *
 create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
 {
@@ -1216,6 +1529,10 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   GtkWidget * ebox;
   GtkWidget * vbox;
   GtkWidget * hbox, * hbox2;
+  GtkWidget * option_menu;
+  GtkWidget * menu;
+  GtkWidget * menuitem;
+  GtkWidget * table;
   GtkWidget * label;
   GtkWidget * pixmap;
 
@@ -1227,16 +1544,19 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   GtkObject * complexity_adj;
   GtkWidget * complexity_hscale;
   GtkObject * framepack_adj;
-  GtkWidget * framepack_hscale;
+  GtkWidget * framepack_spin;
 
   GtkWidget * entry;
 
+  GtkWidget * separator;
+
   GtkAccelGroup * accel_group;
 
   GtkTooltips * tooltips;
 
   GtkStyle * style;
 
+  int i;
   long * l;
 
   dialog = gtk_dialog_new ();
@@ -1267,7 +1587,7 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   main_vbox = GTK_DIALOG(dialog)->vbox;
 
   ebox = gtk_event_box_new ();
-  gtk_box_pack_start (GTK_BOX(main_vbox), ebox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX(main_vbox), ebox, FALSE, TRUE, 0);
   gtk_widget_set_style (ebox, style_bw);
   gtk_widget_show (ebox);
 
@@ -1315,8 +1635,6 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   gtk_box_pack_start (GTK_BOX(main_vbox), notebook, TRUE, TRUE, 4);
   gtk_widget_show (notebook);
 
-  /* Encoding quality */
-
   label = gtk_label_new (_("Speex encoding"));
 
   vbox = gtk_vbox_new (FALSE, 0);
@@ -1324,21 +1642,112 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   gtk_container_set_border_width (GTK_CONTAINER(vbox), 4);
   gtk_widget_show (vbox);
 
+  /* Mode */
+
   hbox = gtk_hbox_new (FALSE, 4);
-  gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);
+  gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, TRUE, 4);
+  gtk_widget_show (hbox);
+
+  label = gtk_label_new (_("Mode:"));
+  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
+
+  option_menu = gtk_option_menu_new ();
+  gtk_box_pack_start (GTK_BOX (hbox), option_menu, TRUE, TRUE, 4);
+  gtk_widget_show (option_menu);
+
+  menu = gtk_menu_new ();
+
+  for (i = 0; mode_choices[i].name != NULL; i++) {
+    menuitem =
+      gtk_menu_item_new_with_label (mode_choices[i].name);
+    gtk_menu_append (GTK_MENU(menu), menuitem);
+    gtk_object_set_user_data (GTK_OBJECT(menuitem),
+			      GINT_TO_POINTER(mode_choices[i].number));
+    gtk_widget_show (menuitem);
+
+    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
+			GTK_SIGNAL_FUNC(speex_encode_options_mode_cb),
+			dialog);
+  }
+  gtk_option_menu_set_menu (GTK_OPTION_MENU(option_menu), menu);
+    
+  gtk_object_set_data (GTK_OBJECT(dialog), "mode_menu", option_menu);
+
+  button = gtk_button_new_with_label (_("Auto"));
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 4);
+  gtk_signal_connect (GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(speex_encode_options_mode_auto_cb),
+		      sample);
+  gtk_widget_show (button);
+  
+  tooltips = gtk_tooltips_new ();
+  gtk_tooltips_set_tip (tooltips, button,
+			_("Automatically select the encoding mode based on "
+			  "the sampling rate of the file."),
+			NULL);
+  
+  separator = gtk_hseparator_new ();
+  gtk_box_pack_start (GTK_BOX(vbox), separator, FALSE, TRUE, 4);
+  gtk_widget_show (separator);
+
+  /* Features */
+
+  option_menu = gtk_option_menu_new ();
+  gtk_box_pack_start (GTK_BOX (vbox), option_menu, FALSE, FALSE, 4);
+  gtk_widget_show (option_menu);
+
+  menu = gtk_menu_new ();
+
+  for (i = 0; feature_choices[i].name != NULL; i++) {
+    menuitem =
+      gtk_menu_item_new_with_label (feature_choices[i].name);
+    gtk_menu_append (GTK_MENU(menu), menuitem);
+    gtk_object_set_user_data (GTK_OBJECT(menuitem),
+			      GINT_TO_POINTER(feature_choices[i].number));
+    gtk_widget_show (menuitem);
+
+    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
+			GTK_SIGNAL_FUNC(speex_encode_options_features_cb),
+			dialog);
+
+    if (feature_choices[i].desc != NULL) {
+      tooltips = gtk_tooltips_new ();
+      gtk_tooltips_set_tip (tooltips, menuitem, feature_choices[i].desc,
+			    NULL);
+    }
+  }
+  gtk_option_menu_set_menu (GTK_OPTION_MENU(option_menu), menu);
+    
+  gtk_object_set_data (GTK_OBJECT(dialog), "features_menu", option_menu);
+
+
+  table = gtk_table_new (3, 2, TRUE);
+  gtk_box_pack_start (GTK_BOX(vbox), table, FALSE, FALSE, 4);
+  gtk_container_set_border_width (GTK_CONTAINER(table), 8);
+  gtk_widget_show (table);
+
+  /* quality */
+
+  hbox = gtk_hbox_new (FALSE, 4);
+  /*gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);*/
+  gtk_table_attach (GTK_TABLE(table), hbox, 0, 1, 0, 1,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  /*gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);*/
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Encoding quality:"));
   gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 4);
   gtk_widget_show (label);
 
+  gtk_object_set_data (GTK_OBJECT (dialog), "quality_label", label);
+
   quality_adj = gtk_adjustment_new (DEFAULT_QUALITY, /* value */
 				    1.0,  /* lower */
-				    10.0, /* upper */
-				    0.001,  /* step incr */
-				    0.001,  /* page incr */
-				    0.001   /* page size */
+				    11.0, /* upper */
+				    1.0,  /* step incr */
+				    1.0,  /* page incr */
+				    1.0   /* page size */
 				    );
 
   {
@@ -1348,15 +1757,21 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
     GtkWidget * vbox_pants;
 
     vbox_pants = gtk_vbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX(hbox), vbox_pants, TRUE, TRUE, 0);
+    /*gtk_box_pack_start (GTK_BOX(hbox), vbox_pants, TRUE, TRUE, 0);*/
+    gtk_table_attach (GTK_TABLE(table), vbox_pants, 1, 2, 0, 1,
+		      GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
     gtk_widget_show (vbox_pants);
 
     quality_hscale = gtk_hscale_new (GTK_ADJUSTMENT(quality_adj));
     gtk_box_pack_start (GTK_BOX (vbox_pants), quality_hscale, TRUE, TRUE, 0);
     gtk_scale_set_draw_value (GTK_SCALE (quality_hscale), TRUE);
+    gtk_scale_set_digits (GTK_SCALE (quality_hscale), 0);
     gtk_widget_set_usize (quality_hscale, gdk_screen_width() / 8, -1);
     gtk_widget_show (quality_hscale);
 
+    gtk_object_set_data (GTK_OBJECT (dialog), "quality_hscale",
+			 quality_hscale);
+
     label = gtk_label_new (NULL);
     gtk_box_pack_start (GTK_BOX(vbox_pants), label, FALSE, FALSE, 0);
     gtk_widget_show (label);
@@ -1371,25 +1786,71 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
 
   gtk_object_set_data (GTK_OBJECT (dialog), "quality_adj", quality_adj);
 
-  /* Variable bitrate mode (VBR) */
+  /* Bit rate */
 
   checkbutton =
-    gtk_check_button_new_with_label (_("Enable variable bitrate mode (VBR)"));
-  gtk_box_pack_start (GTK_BOX(vbox), checkbutton, FALSE, FALSE, 4);
+    gtk_check_button_new_with_label (_("Enable bitrate management"));
+  gtk_table_attach (GTK_TABLE(table), checkbutton, 0, 2, 1, 2,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
   gtk_widget_show (checkbutton);
 
-  gtk_object_set_data (GTK_OBJECT (dialog), "vbr_chb", checkbutton);
+  gtk_signal_connect (GTK_OBJECT(checkbutton), "toggled",
+		      GTK_SIGNAL_FUNC(speex_encode_options_update_cb),
+		      dialog);
+
+  gtk_object_set_data (GTK_OBJECT (dialog), "br_chb", checkbutton);
 
   tooltips = gtk_tooltips_new ();
   gtk_tooltips_set_tip (tooltips, checkbutton,
-			_("This enables variable bitrate mode (VBR)."),
+			_("For non-VBR (constant bitrate) encoding, "
+			  "this sets the maximum bitrate."
+			  "For VBR encoding, this sets the average bitrate."),
 			NULL);
 
-  /* Encoding complexity */
+  label = gtk_label_new (_("Average bitrate"));
+  gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  gtk_widget_show (label);
+
+  gtk_object_set_data (GTK_OBJECT (dialog), "br_label", label);
+
+  hbox = gtk_hbox_new (FALSE, 0);
+  gtk_table_attach (GTK_TABLE(table), hbox, 1, 2, 2, 3,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  gtk_widget_show (hbox);
+
+  entry = gtk_entry_new ();
+  gtk_box_pack_start (GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  gtk_widget_show (entry);
+
+  gtk_object_set_data (GTK_OBJECT (dialog), "br_entry", entry);
+
+  label = gtk_label_new (_("bps"));
+  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 4);
+  gtk_widget_show (label);
+
+  gtk_object_set_data (GTK_OBJECT (dialog), "br_units", label);
+
+  label = gtk_label_new (_("Extra"));
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
+  gtk_container_set_border_width (GTK_CONTAINER(vbox), 4);
+  gtk_widget_show (vbox);
+
+  table = gtk_table_new (2, 2, TRUE);
+  gtk_box_pack_start (GTK_BOX(vbox), table, FALSE, FALSE, 4);
+  /*gtk_container_set_border_width (GTK_CONTAINER(table), 8);*/
+  gtk_widget_show (table);
+
+
+  /* Encoder complexity */
 
   hbox = gtk_hbox_new (FALSE, 4);
-  gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);
+  /*gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);*/
+  gtk_table_attach (GTK_TABLE(table), hbox, 0, 1, 0, 1,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  /*gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);*/
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Encoding complexity:"));
@@ -1398,10 +1859,10 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
 
   complexity_adj = gtk_adjustment_new (DEFAULT_COMPLEXITY, /* value */
 				       1.0,  /* lower */
-				       10.0, /* upper */
-				       0.001,  /* step incr */
-				       0.001,  /* page incr */
-				       0.001   /* page size */
+				       11.0, /* upper */
+				       1.0,  /* step incr */
+				       1.0,  /* page incr */
+				       1.0   /* page size */
 				       );
 
   {
@@ -1411,13 +1872,16 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
     GtkWidget * vbox_pants;
 
     vbox_pants = gtk_vbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX(hbox), vbox_pants, TRUE, TRUE, 0);
+    /*gtk_box_pack_start (GTK_BOX(hbox), vbox_pants, TRUE, TRUE, 0);*/
+    gtk_table_attach (GTK_TABLE(table), vbox_pants, 1, 2, 0, 1,
+		      GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
     gtk_widget_show (vbox_pants);
 
     complexity_hscale = gtk_hscale_new (GTK_ADJUSTMENT(complexity_adj));
     gtk_box_pack_start (GTK_BOX (vbox_pants), complexity_hscale,
 			TRUE, TRUE, 0);
     gtk_scale_set_draw_value (GTK_SCALE (complexity_hscale), TRUE);
+    gtk_scale_set_digits (GTK_SCALE (complexity_hscale), 0);
     gtk_widget_set_usize (complexity_hscale, gdk_screen_width() / 8, -1);
     gtk_widget_show (complexity_hscale);
 
@@ -1430,7 +1894,7 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   gtk_tooltips_set_tip (tooltips, complexity_hscale,
 			_("This sets the encoding speed/quality tradeoff "
 			  "between 0 (faster encoding) "
-			  "and 10 (slower encding)"),
+			  "and 10 (slower encoding)"),
 			NULL);
 
   gtk_object_set_data (GTK_OBJECT (dialog), "complexity_adj", complexity_adj);
@@ -1438,45 +1902,38 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   /* Frames per packet */
 
   hbox = gtk_hbox_new (FALSE, 4);
-  gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);
+  /*gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);*/
+  gtk_table_attach (GTK_TABLE(table), hbox, 0, 1, 1, 2,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  /*gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);*/
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Speex frames per Ogg packet:"));
   gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 4);
   gtk_widget_show (label);
 
+  hbox = gtk_hbox_new (FALSE, 4);
+  /*gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);*/
+  gtk_table_attach (GTK_TABLE(table), hbox, 1, 2, 1, 2,
+		    GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+  /*gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);*/
+  gtk_widget_show (hbox);
+
   framepack_adj = gtk_adjustment_new (DEFAULT_FRAMEPACK, /* value */
 				      1.0,  /* lower */
 				      10.0, /* upper */
-				      0.001,  /* step incr */
-				      0.001,  /* page incr */
-				      0.001   /* page size */
+				      1.0,  /* step incr */
+				      1.0,  /* page incr */
+				      1.0   /* page size */
 				      );
 
-  {
-    /* How sucky ... we create a vbox in order to center the hscale within
-     * its allocation, thus actually lining it up with its label ... 
-     */
-    GtkWidget * vbox_pants;
-
-    vbox_pants = gtk_vbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX(hbox), vbox_pants, TRUE, TRUE, 0);
-    gtk_widget_show (vbox_pants);
-
-    framepack_hscale = gtk_hscale_new (GTK_ADJUSTMENT(framepack_adj));
-    gtk_box_pack_start (GTK_BOX (vbox_pants), framepack_hscale, TRUE, TRUE, 0);
-    gtk_scale_set_draw_value (GTK_SCALE (framepack_hscale), TRUE);
-    gtk_widget_set_usize (framepack_hscale, gdk_screen_width() / 8, -1);
-    gtk_widget_show (framepack_hscale);
-
-    label = gtk_label_new (NULL);
-    gtk_box_pack_start (GTK_BOX(vbox_pants), label, FALSE, FALSE, 0);
-    gtk_widget_show (label);
-  }
+  framepack_spin = gtk_spin_button_new (GTK_ADJUSTMENT(framepack_adj),
+					  1.0, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), framepack_spin, FALSE, FALSE, 0);
+  gtk_widget_show (framepack_spin);
 
   tooltips = gtk_tooltips_new ();
-  gtk_tooltips_set_tip (tooltips, framepack_hscale,
+  gtk_tooltips_set_tip (tooltips, framepack_spin,
 			_("Number of Speex frames to pack into each Ogg "
 			  "packet. Higher values save space at low "
 			  "bitrates."),
@@ -1484,11 +1941,10 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
 
   gtk_object_set_data (GTK_OBJECT (dialog), "framepack_adj", framepack_adj);
 
-
   /* Remember / Reset */
 
   hbox = gtk_hbox_new (FALSE, 4);
-  gtk_box_pack_end (GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
+  gtk_box_pack_start (GTK_BOX(main_vbox), hbox, FALSE, FALSE, 4);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);
   gtk_widget_show (hbox);
 
@@ -1508,7 +1964,7 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
   button = gtk_button_new_with_label (_("Reset"));
   gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, TRUE, 4);
   gtk_signal_connect (GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(speex_encode_options_reset_cb), NULL);
+		      GTK_SIGNAL_FUNC(speex_encode_options_reset_cb), sample);
   gtk_widget_show (button);
 
   tooltips = gtk_tooltips_new ();
@@ -1517,17 +1973,18 @@ create_speex_encoding_options_dialog (sw_sample * sample, char * pathname)
 			NULL);
 
   /* Call the reset callback now to set remembered options */
-  speex_encode_options_reset_cb (button, NULL);
+  speex_encode_options_reset_cb (button, sample);
 
   button = gtk_button_new_with_label (_("Defaults"));
   gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, TRUE, 4);
   gtk_signal_connect (GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(speex_encode_options_default_cb), NULL);
+		      GTK_SIGNAL_FUNC(speex_encode_options_default_cb),
+		      sample);
   gtk_widget_show (button);
 
   tooltips = gtk_tooltips_new ();
   gtk_tooltips_set_tip (tooltips, button,
-			_("Set to default encoding options."),
+			_("Automatically select best encoding options for this file."),
 			NULL);
 
   /* Ogg stream */
