@@ -37,6 +37,7 @@
 #include "callbacks.h"
 #include "interface.h"
 #include "print.h"
+#include "play.h"
 #include "record.h"
 #include "sample.h"
 
@@ -555,6 +556,25 @@ head_controller_create (sw_head * head, GtkWidget * window,
 
 /* head functions */
 
+#define PBOOL(p) ((p) ? "TRUE" : "FALSE" )
+
+int
+head_dump (sw_head * head)
+{
+  printf ("head %p\toffset: %f", head, head->offset);
+  printf ("\tstop:\t%d\n", head->stop_offset);
+  printf ("\tgoing:\t%s\n", PBOOL(head->going));
+  printf ("\trestricted:\t%s\n", PBOOL(head->restricted));
+  printf ("\tlooping:\t%s\n", PBOOL(head->looping));
+  printf ("\tpreviewing:\t%s\n", PBOOL(head->previewing));
+  printf ("\treverse:\t%s\n", PBOOL(head->reverse));
+  printf ("\tmute:\t%s\n", PBOOL(head->mute));
+  printf ("\tgain:\t%f\n", head->gain);
+  printf ("\trate:\t%f\n", head->rate);
+  printf ("\tmix:\t%f\n", head->mix);
+  return 0;
+}
+
 sw_head *
 head_new (sw_sample * sample, sw_head_t head_type)
 {
@@ -755,6 +775,18 @@ head_set_rate (sw_head * h, gfloat rate)
   g_mutex_unlock (h->head_mutex);
 }
 
+void
+head_set_monitor (sw_head * h, gboolean monitor)
+{
+  g_mutex_lock (h->head_mutex);
+
+  h->monitor = monitor;
+
+  g_mutex_unlock (h->head_mutex);
+
+  sample_update_device (h->sample);
+}
+
 static sw_framecount_t
 head_write_unrestricted (sw_head * head, sw_audio_t * buf,
 			 sw_framecount_t count)
