@@ -19,6 +19,7 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <sweep/sweep_i18n.h>
 #include <sweep/sweep_types.h>
 #include "sweep_app.h"
@@ -456,7 +457,7 @@ create_param_set_table (sw_ps_adjuster * ps)
 
 #define ADJUSTER_NUMERIC(T, DIGITS) \
                                                                            \
-	value = (gfloat) pset[i].##T##;                                    \
+	value = (gfloat) pset[i].T;                                    \
                                                                            \
         if (pspec->constraint_type == SW_PARAM_CONSTRAINED_NOT) {          \
           valid = 0;                                                       \
@@ -465,19 +466,19 @@ create_param_set_table (sw_ps_adjuster * ps)
         }                                                                  \
 	                                                                   \
 	if (valid & SW_RANGE_LOWER_BOUND_VALID) {                          \
-	  lower = (gfloat) pspec->constraint.range->lower.##T##;           \
+	  lower = (gfloat) pspec->constraint.range->lower.T;           \
 	} else {                                                           \
 	  lower = G_MINFLOAT;                                              \
 	}                                                                  \
 	                                                                   \
 	if (valid & SW_RANGE_UPPER_BOUND_VALID) {                          \
-	  upper = (gfloat) pspec->constraint.range->upper.##T##;           \
+	  upper = (gfloat) pspec->constraint.range->upper.T;           \
 	} else {                                                           \
 	  upper = G_MAXFLOAT;                                              \
 	}                                                                  \
 	                                                                   \
 	if (valid & SW_RANGE_STEP_VALID) {                                 \
-	  step_inc = (gfloat) pspec->constraint.range->step.##T##;         \
+	  step_inc = (gfloat) pspec->constraint.range->step.T;         \
 	} else if (lower != G_MINFLOAT && upper != G_MAXFLOAT) {           \
           step_inc = (upper - lower) / 100.0;                              \
         } else {                                                           \
