@@ -53,16 +53,22 @@
 #include "play.h"
 #include "head.h"
 #include "driver.h"
-
+#include "preferences.h"
 #include "sample-display.h"
 
+/*#define DEBUG*/
+
 #define SCRUB_SLACKNESS 2.0
 
-GMutex * play_mutex;
+#define USE_MONITOR_KEY "UseMonitor"
 
-static sw_handle * handle = NULL;
+GMutex * play_mutex = NULL;
 
-static GList * active_play_heads = NULL;
+static sw_handle * main_handle = NULL;
+static sw_handle * monitor_handle = NULL;
+
+static GList * active_main_heads = NULL;
+static GList * active_monitor_heads = NULL;
 
 /*static int realoffset = 0;*/
 static sw_sample * prev_sample = NULL;
@@ -70,6 +76,10 @@ static pthread_t player_thread = -1;
 
 static gboolean stop_all = FALSE;
 
+static sw_audio_t * pbuf = NULL, * devbuf = NULL;
+static int pbuf_chans = 0, devbuf_chans = 0;
+
+
 /*
  * update_playmarker ()
  *
@@ -91,7 +101,7 @@ update_playmarker (gpointer data)
   } else if (!head->going) {
 
 #ifdef DEBUG
-    g_print ("update_playmarker: play_mode is STOP\n");
+    g_print ("update_playmarker: !head->going\n");
 #endif
     s->playmarker_tag = 0;
 
@@ -131,266 +141,9 @@ start_playmarker (sw_sample * s)
 
 static struct timeval tv_instant = {0, 0};
 
-#define FORWARDS (1)
-#define BACKWARDS (0)
-
-#ifdef DEVEL_CODE
-/* returns TRUE if it plays all the way to end */
-static gboolean
-play_view(sw_view * view, sw_framecount_t start, sw_framecount_t end,
-	  sw_framecount_t lbound, sw_framecount_t ubound,
-	  gfloat relpitch)
-{
-  sw_sample * s = view->sample;
-  sw_head * head = s->play_head;
-  sw_format * format;
-  fd_set fds;
-  ssize_t n = 0;
-  sw_audio_t * d;
-  gint sbytes, channels;
-  gdouble po = 0.0, p, startf, endf;
-  gdouble momentum_delta = 0.0;
-  gboolean do_smoothing = FALSE;
-  gint i=0, si=0;
-  gint direction = FORWARDS;
-  sw_framecount_t last_user_offset = -1;
-  sw_audio_t pbuf[PBUF_SIZE];
-
-  if (handle == NULL) return FALSE;
-
-  d = (sw_audio_t *)s->sounddata->data;
-
-  sbytes = 2;
- 
-  format = s->sounddata->format;
-  channels = format->channels;
-
-  /* compensate for drivers unable to do required sampling rate */
-  relpitch *= (format->rate / handle->driver_rate);
-  
-  startf = (gdouble)(lbound);
-  endf = (gdouble)(ubound-channels); /* linear interp wants to look ahead
-				      * one frame */
-
-  if (head->reverse) {
-    head->offset = (gdouble)end;
-  } else {
-    head->offset = (gdouble)start;
-  }
-
-  po = (gdouble)head->offset;
-
-  memset (pbuf, 0, sizeof (pbuf));
-
-  while ((head->reverse ? (po >= startf) : (po < endf)) &&
-	 sample_bank_contains (s) && head->going) {
-
-    FD_ZERO (&fds);
-    FD_SET (handle->driver_fd, &fds);
-    
-    if (select (handle->driver_fd+1, NULL, &fds, NULL, &tv_instant) == 0);
-
-    /*    memset (pbuf, 0, sizeof (pbuf));*/
-
-    for (i = 0; i < PBUF_SIZE; i++) {
-
-      if (head->mute || s->user_offset == last_user_offset ||
-	  po < lbound || po >= ubound) {
-	pbuf[i] = 0;
-	if (channels == 1 && handle->driver_channels == 2) {
-	  pbuf[++i] = 0;
-	}
-
-      } else {
-
-	switch (channels) {
-	case 1:
-	  switch (handle->driver_channels) {
-	  case 1:
-	    /* Mono to mono */
-	    si = (int)floor(po);
-	    p = po - (gdouble)si;
-	    pbuf[i] = head->gain * (d[si] * p + d[si+channels] * (1 - p));
-	    break;
-	  case 2:
-	    /* Mix mono data up to stereo device */
-	    si = (int)floor(po);
-	    p = po - (gdouble)si;
-	    pbuf[i] = head->gain *
-	      (d[si] * p + d[si+channels] * (1 - p));
-	    pbuf[i+1] = pbuf[i];
-	    i++;
-	    break;
-	  default:
-	    break;
-	  }
-	  break;
-	default:
-	  switch (handle->driver_channels) {
-	  case 1:
-	    /* Mix multichannel data down to mono device */
-	    si = (int)floor(po);
-	    p = po - (gdouble)si;
-	    si *= channels;
-	    pbuf[i] = head->gain * 0.5 * ((d[si] + d[si+1]) * p +
-	       (d[si+channels] + d[si+1+channels]) * (1 - p));
-	    break;
-	  case 2:
-	    /* Stereo/Multichannel to stereo */
-	    si = (int)floor(po);
-	    p = po - (gdouble)si;
-	    si *= channels;
-	    pbuf[i] = head->gain * (d[si] * p + d[si+channels] * (1 - p));
-	    i++; si++;
-	    pbuf[i] = head->gain * (d[si] * p + d[si+channels] * (1 - p));
-	    break;
-	  default:
-	    break;
-	  }
-      
-	  break;
-	}
-
-#if 1
-	/* If this data came about from scrubbing, it is probably a bit
-	 * noisy; run it through a simple smoothing filter:
-	 *
-	 *   x[n] = ( x[n] + 2 * x[n-1] + 3 * x[n-2] ) / 6
-	 */
-	if (do_smoothing) {
-	  int iL1, iL2;
-	  int iR1, iR2;
-	  
-	  switch (handle->driver_channels) {
-	  case 1:
-	    iL1 = (i - 1 + PBUF_SIZE) % PBUF_SIZE;
-	    iL2 = (i - 2 + PBUF_SIZE) % PBUF_SIZE;
-	    pbuf[i] += pbuf[i] * 2.0;
-	    pbuf[i] += pbuf[iL1]*3.0 + pbuf[iL2]*4.0;
-	    pbuf[i] /= 4.0;
-	    break;
-	  case 2:
-	    iL1 = (i - 2 + PBUF_SIZE) % PBUF_SIZE;
-	    iL2 = (i - 4 + PBUF_SIZE) % PBUF_SIZE;
-	    iR1 = (i - 3 + PBUF_SIZE) % PBUF_SIZE;
-	    iR2 = (i - 5 + PBUF_SIZE) % PBUF_SIZE;
-	    pbuf[i] += pbuf[i] * 2.0;
-	    pbuf[i] += pbuf[iL1] * 3.0 + pbuf[iL2] * 4.0;
-	    pbuf[i] /= 4.0;
-	    pbuf[i-1] += pbuf[i-1] * 2.0;
-	    pbuf[i-1] += pbuf[iR1] * 3.0 + pbuf[iR2] * 4.0;
-	    pbuf[i-1] /= 4.0;
-	  }
-	  do_smoothing = FALSE;
-	}
-#else
-	
-#endif
-      }
-
-
-      if (s->by_user) {
-	if (head->scrubbing) {
-	  gdouble old_po = po;
-	  gdouble play_delta;
-
-	  if (s->user_offset < (int)po) {
-	    direction = BACKWARDS;
-	  } else {
-	    direction = FORWARDS;
-	  }
-
-#if 0
-	  po = (po + (SCRUB_SLACKNESS-1.0)*(gdouble)s->user_offset) /
-	    SCRUB_SLACKNESS;
-#elif 1
-	  po = (po*(SCRUB_SLACKNESS-1.0) + (gdouble)s->user_offset) /
-	    SCRUB_SLACKNESS;
-#else
-	  po = (gdouble)s->user_offset;
-#endif
-
-	  momentum_delta = (po - old_po)/8;
-	  momentum_delta = CLAMP(momentum_delta, -10, +10);
-
-#if 0	      
-	  play_delta = view->rate * relpitch;
-	  play_delta += momentum_delta;
-#else
-	  play_delta = momentum_delta;
-#endif
-
-	  if (direction == FORWARDS && !head->reverse)
-	    po += play_delta;
-	  else if (direction == BACKWARDS && head->reverse)
-	    po -= play_delta;
-
-	  head->offset = (int)po;
-
-	} else {
-	  head->offset = s->user_offset;
-	  po = (gdouble)head->offset;
-	  direction = FORWARDS;
-	  momentum_delta = 0;
-	}
-
-	last_user_offset = s->user_offset;
-	s->by_user = FALSE;
-
-	do_smoothing = TRUE;
-      } else {
-	gdouble play_delta;
-
-	play_delta = s->rate * relpitch;
-
-	if (head->reverse) {
-	  po -= play_delta;
-	} else {
-	  po += play_delta;
-	}
-
-	po += momentum_delta;
-
-	head->offset = (int)po;
-
-	momentum_delta *= 0.99;
-      }
-
-      if (po < startf || po >= endf) break;
-    } /* for (i) */
-
-    /* Only write if still playing */
-    if (head->going) {
-
-      n = device_write (handle, pbuf, PBUF_SIZE, head->offset);
-
-      g_mutex_lock (s->play_mutex);
-
-      head->realoffset = device_offset (handle);
-      if (head->realoffset == -1) {
-	head->realoffset = head->offset;
-      }
-
-      head->offset = head->realoffset;
-      
-      if (s->by_user /* && s->play_scrubbing */) {
-	head->offset = s->user_offset;
-      } else {
-	s->user_offset = head->realoffset;
-      }
-
-      g_mutex_unlock (s->play_mutex);
-    }
-
-  }
-
-  return (head->reverse ? (po <= startf) : (po >= endf));
-}
-#endif /* play_view: DEVEL_CODE, soon obsolete */
-
 static sw_framecount_t
 head_read_unrestricted (sw_head * head, sw_audio_t * buf,
-			sw_framecount_t count)
+			sw_framecount_t count, int driver_rate)
 {
   sw_sample * sample = head->sample;
   sw_sounddata * sounddata = sample->sounddata;
@@ -413,7 +166,7 @@ head_read_unrestricted (sw_head * head, sw_audio_t * buf,
   po = head->offset;
 
   /* compensate for sampling rate of driver */
-  relpitch = (gfloat)((gdouble)f->rate / (gdouble)handle->driver_rate);
+  relpitch = (gfloat)((gdouble)f->rate / (gdouble)driver_rate);
   
   for (i = 0; i < count; i++) {
     if (head->mute || sample->user_offset == last_user_offset) {
@@ -537,14 +290,17 @@ head_read_unrestricted (sw_head * head, sw_audio_t * buf,
 }
 
 sw_framecount_t
-head_read (sw_head * head, sw_audio_t * buf, sw_framecount_t count)
+head_read (sw_head * head, sw_audio_t * buf, sw_framecount_t count,
+	   int driver_rate)
 {
   sw_sample * sample = head->sample;
   sw_sounddata * sounddata = sample->sounddata;
   sw_format * f = sounddata->format;
+  sw_framecount_t head_offset;
   sw_framecount_t remaining = count, written = 0, n = 0;
+  sw_framecount_t delta, bound;
   GList * gl;
-  sw_sel * sel;
+  sw_sel * sel, * osel;
 
   while (head->going && remaining > 0) {
     n = 0;
@@ -554,32 +310,113 @@ head_read (sw_head * head, sw_audio_t * buf, sw_framecount_t count)
 
       if (g_list_length (sounddata->sels) == 0) {
 	g_mutex_unlock (sounddata->sels_mutex);
-	goto zero_pad;
+
+	if (head->previewing && !head->scrubbing) {
+	  if (head->reverse) {
+	    n = MIN (remaining, head->offset - head->stop_offset);
+	  } else {
+	    n = MIN (remaining, head->stop_offset - head->offset);
+	  }
+	  goto got_n;
+
+	} else {
+	  goto zero_pad;
+	}
       }
 
-      /* Find selection region that offset is or should be in */
-      if (head->reverse) {
-	for (gl = g_list_last (sounddata->sels); gl; gl = gl->prev) {
-	  sel = (sw_sel *)gl->data;
+      if (head->previewing && !head->scrubbing) {
+	/* Find selection region that offset is or should be playing to */
+	if (head->reverse) {
+	  osel = NULL;
+	  for (gl = g_list_last (sounddata->sels); gl; gl = gl->prev) {
+	    sel = (sw_sel *)gl->data;
+	    
+	    if (osel && ((sw_framecount_t)head->offset > osel->sel_start))
+	      head->offset = osel->sel_start;
 
-	  if (head->offset > sel->sel_end)
-	    head->offset = sel->sel_end;
-	
-	  if (head->offset > sel->sel_start) {
-	    n = MIN (remaining, head->offset - sel->sel_start);
-	    break;
+	    osel = sel;
+
+	    head_offset = (sw_framecount_t)head->offset;
+	    
+	    if (head_offset > sel->sel_end) {
+	      n = MIN (remaining, head_offset - sel->sel_end);
+	      break;
+	    }
+	  }
+
+	  /* If now at start of first selection region ... */
+	  if (gl == NULL && osel != NULL) {
+	    if ((sw_framecount_t)head->offset > osel->sel_start)
+	      head->offset = osel->sel_start;
+
+	    head_offset = (sw_framecount_t)head->offset;
+	    
+	    /* continue 1 second */
+	    delta = time_to_frames (sounddata->format, 1.0);
+	    bound = MAX((osel->sel_start - delta), 0);
+	    if (head_offset > bound) {
+	      n = MIN (remaining, head_offset - bound);
+	    }
+	  }
+
+	} else {
+	  osel = NULL;
+	  for (gl = sounddata->sels; gl; gl = gl->next) {
+	    sel = (sw_sel *)gl->data;
+	    
+	    if (osel && ((sw_framecount_t)head->offset < osel->sel_end))
+	      head->offset = osel->sel_end;
+
+	    osel = sel;
+	    
+	    head_offset = (sw_framecount_t)head->offset;
+
+	    if (head_offset < sel->sel_start) {
+	      n = MIN (remaining, sel->sel_start - head_offset);
+	      break;
+	    }
+	  }
+	  
+	  /* If now at end of last selection region ... */
+	  if (gl == NULL && osel != NULL) {
+	    if ((sw_framecount_t)head->offset < osel->sel_end)
+	      head->offset = osel->sel_end;
+
+	    head_offset = head->offset;
+
+	    /* continue 1 second */
+	    delta = time_to_frames (sounddata->format, 1.0);
+	    bound = MIN ((osel->sel_end + delta), sounddata->nr_frames);
+	    if (head_offset < bound) {
+	      n = MIN (remaining, bound - head_offset);
+	    }
 	  }
 	}
       } else {
-	for (gl = sounddata->sels; gl; gl = gl->next) {
-	  sel = (sw_sel *)gl->data;
-
-	  if (head->offset < sel->sel_start)
-	    head->offset = sel->sel_start;
-	
-	  if (head->offset < sel->sel_end) {
-	    n = MIN (remaining, sel->sel_end - head->offset);
-	    break;
+	/* Find selection region that offset is or should be in */
+	if (head->reverse) {
+	  for (gl = g_list_last (sounddata->sels); gl; gl = gl->prev) {
+	    sel = (sw_sel *)gl->data;
+	    
+	    if ((sw_framecount_t)head->offset > sel->sel_end)
+	      head->offset = sel->sel_end;
+	    
+	    if ((sw_framecount_t)head->offset > sel->sel_start) {
+	      n = MIN (remaining, (sw_framecount_t)head->offset - sel->sel_start);
+	      break;
+	    }
+	  }
+	} else {
+	  for (gl = sounddata->sels; gl; gl = gl->next) {
+	    sel = (sw_sel *)gl->data;
+	    
+	    if ((sw_framecount_t)head->offset < sel->sel_start)
+	      head->offset = sel->sel_start;
+	    
+	    if ((sw_framecount_t)head->offset < sel->sel_end) {
+	      n = MIN (remaining, sel->sel_end - (sw_framecount_t)head->offset);
+	      break;
+	    }
 	  }
 	}
       }
@@ -587,15 +424,27 @@ head_read (sw_head * head, sw_audio_t * buf, sw_framecount_t count)
       g_mutex_unlock (sounddata->sels_mutex);
 
     } else { /* unrestricted */
-      if (head->reverse) {
-	n = MIN (remaining, head->offset);
+      if (head->previewing && !head->scrubbing) {
+	if (head->reverse) {
+	  n = MIN (remaining, (sw_framecount_t)head->offset - head->stop_offset);
+	} else {
+	  n = MIN (remaining, head->stop_offset - (sw_framecount_t)head->offset);
+	}
       } else {
-	n = MIN (remaining, sounddata->nr_frames - head->offset);
+	if (head->reverse) {
+	  n = MIN (remaining, (sw_framecount_t)head->offset);
+	} else {
+	  n = MIN (remaining, sounddata->nr_frames - (sw_framecount_t)head->offset);
+	}
       }
     }
 
+  got_n:
+
     if (n == 0) {
-      if (!head->restricted || sounddata->sels == NULL) {
+      if (head->previewing) {
+	head->offset = head->stop_offset;
+      } else if (!head->restricted || sounddata->sels == NULL) {
 	head->offset = head->reverse ? sounddata->nr_frames : 0;
       } else {
 	g_mutex_lock (sounddata->sels_mutex);
@@ -611,13 +460,24 @@ head_read (sw_head * head, sw_audio_t * buf, sw_framecount_t count)
 	g_mutex_unlock (sounddata->sels_mutex);
       }
 
-      if (!head->looping) {
+      if (!head->looping || head->previewing) {
 	head->going = FALSE;
       }
     } else {
-      written += head_read_unrestricted (head, buf, n);
+      if (n < 0) {
+#ifdef DEBUG
+	printf ("n = %d\n", n);
+#endif
+	head->going = FALSE;
+#ifdef DEUBG
+      } else if (n > count) {
+	printf ("n = %d \t>\tcount = %d\n", n, count);
+#endif
+      } else {
+      written += head_read_unrestricted (head, buf, n, driver_rate);
       buf += (int)frames_to_samples (f, n);
       remaining -= n;
+      }
     }
   }
 
@@ -640,6 +500,7 @@ head_init_playback (sw_sample * s)
   GList * gl;
   sw_sel * sel;
   sw_framecount_t sels_start, sels_end;
+  sw_framecount_t delta;
 
   /*  g_mutex_lock (s->play_mutex);*/
 
@@ -650,25 +511,61 @@ head_init_playback (sw_sample * s)
     head->delta = head->rate * s->rate;
   }
 
+
   if (head->restricted) {
     g_mutex_lock (s->sounddata->sels_mutex);
 
-    gl = s->sounddata->sels;
-    sel = (sw_sel *)gl->data;
-    sels_start = sel->sel_start;
-
-    gl = g_list_last (s->sounddata->sels);
-  
-    sel = (sw_sel *)gl->data;
-    sels_end = sel->sel_end;
-    g_mutex_unlock (s->sounddata->sels_mutex);
-
-    if (head->reverse && head->offset <= sels_start) {
-      head_set_offset (head, sels_end);
-    } else if (!head->reverse && head->offset >= sels_end) {
-      head_set_offset (head, sels_start);
+    if ((gl = s->sounddata->sels) != NULL) {
+      sel = (sw_sel *)gl->data;
+      sels_start = sel->sel_start;
+      
+      gl = g_list_last (s->sounddata->sels);
+      
+      sel = (sw_sel *)gl->data;
+      sels_end = sel->sel_end;
+      g_mutex_unlock (s->sounddata->sels_mutex);
+      
+      if (head->previewing) {
+	/* preroll 1 second */
+	delta = time_to_frames (s->sounddata->format, 1.0);
+	
+	if (head->reverse) {
+	  head_set_offset (head, sels_end + delta);
+	} else {
+	  head_set_offset (head, sels_start - delta);
+	}
+	
+	head_set_offset (head,
+			 CLAMP((sw_framecount_t)head->offset, 0, s->sounddata->nr_frames));
+      } else {
+	if (head->reverse && head->offset <= sels_start) {
+	  head_set_offset (head, sels_end);
+	} else if (!head->reverse && head->offset >= sels_end) {
+	  head_set_offset (head, sels_start);
+	}
+      }
+    } else if (head->previewing) {
+	/* preroll 1 second */
+	delta = time_to_frames (s->sounddata->format, 1.0);
+	
+	if (head->reverse) {
+	  head_set_offset (head, head->offset + delta);
+	} else {
+	  head_set_offset (head, head->offset - delta);
+	}
+	
+	head_set_offset (head,
+			 CLAMP(head->offset, 0, s->sounddata->nr_frames));
     }
   } else {
+
+    if (head->previewing) {
+      /* preroll 1 second */
+      delta = time_to_frames (s->sounddata->format, 1.0);
+      if (head->reverse) head_set_offset (head, head->offset + delta);
+      else head_set_offset (head, head->offset - delta);
+    }
+
     if (head->reverse && head->offset <= 0) {
       head_set_offset (head,  s->sounddata->nr_frames);
     } else if (!head->reverse && head->offset >= s->sounddata->nr_frames) {
@@ -727,19 +624,25 @@ channel_convert_adding (sw_audio_t * src, int src_channels,
 }
 
 static void
-prepare_play_head (sw_head * head)
+play_head_update_device (sw_head * head)
 {
-  sw_sample * s = head->sample;
-
   g_mutex_lock (play_mutex);
+  g_mutex_lock (head->head_mutex);
 
-  if (g_list_find (active_play_heads, head) == 0) {
-    active_play_heads = g_list_append (active_play_heads, head);
+  if (head->monitor) {
+    if (g_list_find (active_monitor_heads, head) == 0) {
+      active_monitor_heads = g_list_append (active_monitor_heads, head);
+    }
+    active_main_heads = g_list_remove (active_main_heads, head);
+  } else {
+    if (g_list_find (active_main_heads, head) == 0) {
+      active_main_heads = g_list_append (active_main_heads, head);
+    }
+    active_monitor_heads = g_list_remove (active_monitor_heads, head);
   }
 
+  g_mutex_unlock (head->head_mutex);
   g_mutex_unlock (play_mutex);
-
-  head_init_playback (s);
 }
 
 #ifdef RECORD_DEMO_FILES
@@ -752,132 +655,239 @@ generate_demo_filename (void)
 
 #define PSIZ 64
 
+static void
+prepare_to_play_heads (GList * heads, sw_handle * handle)
+{
+  sw_head * head;
+  sw_format * f;
+  fd_set fds;
+
+  GList * gl;
+
+  if ((gl = heads) != NULL) {
+    head = (sw_head *)gl->data;
+
+    f = head->sample->sounddata->format;
+    
+    if (f->channels > pbuf_chans) {
+      pbuf = g_realloc (pbuf, PSIZ * f->channels * sizeof (sw_audio_t));
+      pbuf_chans = f->channels;
+    }
+  }
+
+  FD_ZERO (&fds);
+  FD_SET (handle->driver_fd, &fds);
+  
+  if (select (handle->driver_fd+1, &fds, NULL, NULL, &tv_instant) == 0);
+  
+  return;
+}
+
+static void
+play_heads (GList ** heads, sw_handle * handle)
+{
+  sw_sample * s;
+  sw_head * head;
+  sw_format * f;
+  sw_framecount_t n;
+
+  GList * gl, * gl_next;
+
+  if (*heads == NULL) return;
+
+  n = PSIZ;
+  
+  for (gl = *heads; gl; gl = gl_next) {
+
+    g_mutex_lock (play_mutex);
+    head = (sw_head *)gl->data;
+    gl_next = gl->next;
+    g_mutex_unlock (play_mutex);
+
+    if (!head) {
+      /* XXX: wtf??? */
+      return;
+    } else if (!head->going || !sample_bank_contains (head->sample)) {
+      g_mutex_lock (play_mutex);
+      *heads = g_list_remove (*heads, head);
+      g_mutex_unlock (play_mutex);
+    } else {
+      s = head->sample;
+      f = s->sounddata->format;
+	  
+      if (f->channels > pbuf_chans) {
+	pbuf = g_realloc (pbuf, n * f->channels * sizeof (sw_audio_t));
+	pbuf_chans = f->channels;
+      }
+
+      head_read (head, pbuf, n, handle->driver_rate);
+	  
+#if 0
+      if (f->channels != handle->driver_channels) {
+	channel_convert (pbuf, f->channels, devbuf,
+			 handle->driver_channels, n);
+      }
+#else
+      channel_convert_adding (pbuf, f->channels, devbuf,
+			      handle->driver_channels, n);
+#endif
+	
+      /* XXX: store the head->offset NOW for device_offset referencing */
+	  
+      g_mutex_lock (s->play_mutex);
+	  
+#if 0
+      head->realoffset = head->offset;
+#else
+      head->realoffset = device_offset (handle);
+      if (head->realoffset == -1) {
+	head->realoffset = head->offset;
+      }
+#endif
+	
+      head->offset = head->realoffset;
+	  
+      if (s->by_user /* && s->play_scrubbing */) {
+	/*head->offset = s->user_offset;*/
+      } else {
+	if (!head->scrubbing) s->user_offset = head->realoffset;
+      }
+	  
+      /*	if (!head->going) active = FALSE;*/
+	  
+      g_mutex_unlock (s->play_mutex);
+    }
+  }
+
+  return;
+}
+
 /* how many inactive writes to do before closing */
 #define INACTIVE_TIMEOUT 256
 
-static void
-play_head (sw_head * nymhead)
+static gboolean
+monitor_active (void)
 {
-  sw_sample * s;
-  sw_format * f;
-  sw_framecount_t n, count;
-  fd_set fds;
-  sw_audio_t * pbuf, * devbuf;
+  int * use_monitor;
+
+  use_monitor = prefs_get_int (USE_MONITOR_KEY);
+
+  if (use_monitor == NULL) return 0;
+  else return (*use_monitor != 0);
+}
+
+static void
+play_active_heads (void)
+{
+  sw_framecount_t count;
   int inactive_writes = 0;
-  GList * gl, * gl_next;
+  GList * gl;
   sw_head * head;
-  int pbuf_chans;
+  sw_format * f;
+  int max_driver_chans = 0;
+
+  gboolean use_monitor;
 
 #ifdef RECORD_DEMO_FILES
   gchar * filename;
-  SNDFILE * sndfile;
+  SNDFILE * sndfile = NULL;
   SF_INFO sfinfo;
 #endif
 
-  if (!(gl = active_play_heads)) return;
-  
-  head = (sw_head *)gl->data;
+  if (!active_main_heads && !active_monitor_heads) return;
 
-  f = head->sample->sounddata->format;
+  use_monitor = (monitor_handle != NULL);
 
-  device_setup (handle, f);
+  if (use_monitor) {
+    if ((gl = active_monitor_heads)) {
+      head = (sw_head *)gl->data;
+      f = head->sample->sounddata->format;
+      
+      device_setup (monitor_handle, f);
+    } else if ((gl = active_main_heads)) {
+      head = (sw_head *)gl->data;
+      f = head->sample->sounddata->format;
+      
+      device_setup (monitor_handle, f);
+    }
+  }
 
-  pbuf = g_malloc (PSIZ * f->channels * sizeof (sw_audio_t));
-  pbuf_chans = f->channels;
+  if ((gl = active_main_heads)) {
+    head = (sw_head *)gl->data;
+    f = head->sample->sounddata->format;
 
-  devbuf = g_malloc (PSIZ * handle->driver_channels * sizeof (sw_audio_t));
+    device_setup (main_handle, f);
 
-#ifdef RECORD_DEMO_FILES
-  filename = generate_demo_filename ();
-  sfinfo.samplerate = f->rate;
-  sfinfo.channels = handle->driver_channels;
-  sfinfo.format = SF_FORMAT_AU | SF_FORMAT_FLOAT | SF_ENDIAN_CPU;
-  sndfile = sf_open (filename, SFM_WRITE, &sfinfo);
-  if (sndfile == NULL) sf_perror (NULL);
-  else printf ("Writing to %s\n", filename);
+#ifdef RECORD_DEMO_FILES    
+    filename = generate_demo_filename ();
+    sfinfo.samplerate = f->rate;
+    sfinfo.channels = handle->driver_channels;
+    sfinfo.format = SF_FORMAT_AU | SF_FORMAT_FLOAT | SF_ENDIAN_CPU;
+    sndfile = sf_open (filename, SFM_WRITE, &sfinfo);
+    if (sndfile == NULL) sf_perror (NULL);
+    else printf ("Writing to %s\n", filename);
 #endif
+  } else if ((gl = active_monitor_heads)) {
+    head = (sw_head *)gl->data;
+    f = head->sample->sounddata->format;
+
+    device_setup (main_handle, f);
+  } else {
+    return;
+  }
+
+  if (use_monitor) {
+    max_driver_chans = MAX (main_handle->driver_channels,
+			    monitor_handle->driver_channels);
+  } else {
+    max_driver_chans = main_handle->driver_channels;
+  }
+
+  if (max_driver_chans > devbuf_chans) {
+    devbuf = g_realloc (devbuf,
+			PSIZ * max_driver_chans * sizeof (sw_audio_t));
+    devbuf_chans = max_driver_chans;
+  }
 
   while (!stop_all && inactive_writes < INACTIVE_TIMEOUT) {
-    FD_ZERO (&fds);
-    FD_SET (handle->driver_fd, &fds);
-      
-    if (select (handle->driver_fd+1, &fds, NULL, NULL, &tv_instant) == 0);
 
-    n = PSIZ;
-
-    count = n * handle->driver_channels;
-
-    memset (devbuf, 0, PSIZ * handle->driver_channels * sizeof (sw_audio_t));
-
-    if (active_play_heads == NULL) {
+    if (active_main_heads == NULL && active_monitor_heads == NULL) {
       inactive_writes++;
     } else {
       inactive_writes = 0;
-
-      for (gl = active_play_heads; gl; gl = gl_next) {
-	head = (sw_head *)gl->data;
-	s = head->sample;
-	gl_next = gl->next;
-
-	if (!head->going || !sample_bank_contains (head->sample)) {
-	  g_mutex_lock (play_mutex);
-	  active_play_heads = g_list_remove (active_play_heads, head);
-	  g_mutex_unlock (play_mutex);
-	} else {
-
-	  f = s->sounddata->format;
-	  
-	  if (f->channels > pbuf_chans) {
-	    pbuf = g_realloc (pbuf, PSIZ * f->channels * sizeof (sw_audio_t));
-	    pbuf_chans = f->channels;
-	  }
-	  
-	  head_read (head, pbuf, n);
-	  
-#if 0
-	  if (f->channels != handle->driver_channels) {
-	    channel_convert (pbuf, f->channels, devbuf,
-			     handle->driver_channels, n);
-	  }
-#else
-	  channel_convert_adding (pbuf, f->channels, devbuf,
-				  handle->driver_channels, n);
-
-#endif
-	
-	  /* XXX: store the head->offset NOW for device_offset referencing */
-	  
-	  g_mutex_lock (s->play_mutex);
-	  
-#if 0
-	  head->realoffset = head->offset;
-#else
-	  head->realoffset = device_offset (handle);
-	  if (head->realoffset == -1) {
-	    head->realoffset = head->offset;
-	  }
-#endif
-	
-	  head->offset = head->realoffset;
-	  
-	  if (s->by_user /* && s->play_scrubbing */) {
-	    /*head->offset = s->user_offset;*/
-	  } else {
-	    if (!head->scrubbing) s->user_offset = head->realoffset;
-	  }
-	  
-	  /*	if (!head->going) active = FALSE;*/
-	  
-	  g_mutex_unlock (s->play_mutex);
-	}
-      }
     }
-    
-    device_write (handle, devbuf, count, -1 /* offset reference */);  
+
+    g_mutex_lock (play_mutex);
+    if (use_monitor) {
+      prepare_to_play_heads (active_monitor_heads, monitor_handle);
+      prepare_to_play_heads (active_main_heads, main_handle);
+    } else {
+      prepare_to_play_heads (active_monitor_heads, main_handle);
+      prepare_to_play_heads (active_main_heads, main_handle);
+    }
+    g_mutex_unlock (play_mutex);
+
+    if (use_monitor) {
+      count = PSIZ * monitor_handle->driver_channels;
+      memset (devbuf, 0, count * sizeof (sw_audio_t));
+      play_heads (&active_monitor_heads, monitor_handle);
+      device_write (monitor_handle, devbuf, count, -1 /* offset reference */);  
+      
+      count = PSIZ * main_handle->driver_channels;
+      memset (devbuf, 0, count * sizeof (sw_audio_t));
+      play_heads (&active_main_heads, main_handle);
+      device_write (main_handle, devbuf, count, -1 /* offset reference */);
+    } else {
+      count = PSIZ * main_handle->driver_channels;
+      memset (devbuf, 0, count * sizeof (sw_audio_t));
+      play_heads (&active_monitor_heads, main_handle);
+      play_heads (&active_main_heads, main_handle);
+      device_write (main_handle, devbuf, count, -1 /* offset reference */);  
+    }
 
 #ifdef RECORD_DEMO_FILES
-    if (sndfile) sf_writef_float (sndfile, devbuf,
-				  count / handle->driver_channels);
+    if (sndfile)
+      sf_writef_float (sndfile, devbuf, count / main_handle->driver_channels);
 #endif
   }
 
@@ -889,8 +899,13 @@ play_head (sw_head * nymhead)
   }
 #endif
 
-  device_reset (handle);
-  device_close (handle);
+  if (use_monitor) {
+    device_reset (monitor_handle);
+    device_close (monitor_handle);
+  }
+
+  device_reset (main_handle);
+  device_close (main_handle);
 
 #ifdef RECORD_DEMO_FILES
   if (sndfile) {
@@ -908,9 +923,16 @@ ensure_playing (void)
   sw_handle * h;
 
   if (player_thread == -1) {
-    if ((h = device_open (O_WRONLY)) != NULL) {
-      handle = h;
-      pthread_create (&player_thread, NULL, (void *) (*play_head), NULL);
+    if ((h = device_open (0, O_WRONLY)) != NULL) {
+      main_handle = h;
+      if (monitor_active()) {
+	monitor_handle = device_open (1, O_WRONLY);
+      } else {
+	monitor_handle = NULL;
+      }
+
+      pthread_create (&player_thread, NULL, (void *) (*play_active_heads),
+		      NULL);
       return TRUE;
     } else {
       return FALSE;
@@ -925,7 +947,8 @@ sample_play (sw_sample * sample)
 {
   sw_head * head = sample->play_head;
 
-  prepare_play_head (head);
+  play_head_update_device (head);
+  head_init_playback (sample);
 
   head_set_going (head, TRUE);
 
@@ -939,6 +962,16 @@ sample_play (sw_sample * sample)
   }
 }
 
+void
+sample_update_device (sw_sample * sample)
+{
+  sw_head * head = sample->play_head;
+
+  if (!head->going) return;
+
+  play_head_update_device (head);
+}
+
 void
 play_view_all (sw_view * view)
 {
@@ -953,20 +986,6 @@ play_view_all (sw_view * view)
   head_set_restricted (head, FALSE);
 
   sample_play (s);
-#if 0
-  prepare_play_head (head);
-
-  head_set_going (head, TRUE);
-
-  sample_refresh_playmode (s);
-
-  if (ensure_playing()) {
-    start_playmarker (s);
-  } else {
-    head_set_going (head, FALSE);
-    sample_refresh_playmode (s);
-  }
-#endif
 }
 
 void
@@ -989,310 +1008,47 @@ play_view_sel (sw_view * view)
   head_set_restricted (head, TRUE);
 
   sample_play (s);
-
-#if 0
-  prepare_play_head (head);
-
-  head_set_going (head, TRUE);
-
-  sample_refresh_playmode (s);
-
-  if (ensure_playing()) {
-    start_playmarker (s);
-  } else {
-    head_set_going (head, FALSE);
-    sample_refresh_playmode (s);
-  }
-#endif
 }
 
-#ifdef DEVEL_CODE
-static void
-pvpreview_cursor (sw_view * view)
-{
-  sw_sample * s = view->sample;
-  sw_head * head = s->play_head;
-  sw_framecount_t delta, preview_start, preview_end;
-
-  device_setup (handle, s->sounddata->format);
-
-  head_init_playback (s);
-  
-  delta = time_to_frames (s->sounddata->format, 1.0); /* 1 second */ 
-
-  preview_start = head->offset - delta;
-  preview_start = MAX (preview_start, 0);
-
-  preview_end = head->offset + delta;
-  preview_end = MIN (preview_end, s->sounddata->nr_frames);
-
-  if (play_view (view, preview_start, preview_end,
-		 preview_start, preview_end, 1.0)) {
-    g_mutex_lock (head->head_mutex);
-    head->offset = head->stop_offset;
-    g_mutex_unlock (head->head_mutex);
-  }
-
-  device_drain (handle);
-
-  HEAD_SET_GOING (head, FALSE);
-
-  device_reset (handle);
-  device_close (handle);
-}
-#endif
-
-void
-play_preview_cursor (sw_view * view)
-{
-#if 0
-  sw_handle * h;
-
-  sw_sample * s = view->sample;
-
-  stop_playback (prev_sample);
-
-  head_set_going (head, TRUE);
-  sample_refresh_playmode (s);
-  sample_set_stop_offset (s);
-  sample_set_previewing (s, TRUE);
-
-  prev_sample = s;
-
-  if ((h = device_open(O_WRONLY)) != NULL) {
-    handle = h;
-#ifdef DEBUG
-    if (player_thread != -1) {
-      g_print ("Warning: runaway player thread!\n");
-    }
-#endif
-    pthread_create (&player_thread, NULL, (void *) (*pvpreview_cursor),
-		    view);
-    start_playmarker (s);
-  } else {
-    sample_set_playmode (s, SWEEP_TRANSPORT_STOP);
-  }
-#endif
-}
-
-#ifdef DEVEL_CODE
-static void
-pvpreroll (sw_view * view)
-{
-  sw_sample * s = view->sample;
-  sw_head * head = s->play_head;
-  sw_framecount_t delta, preroll_start, preroll_end;
-
-  device_setup (handle, s->sounddata->format);
-
-  head_init_playback (s);
-
-  delta = time_to_frames (s->sounddata->format, 1.0); /* preroll 1 second */
- 
-  preroll_start = head->reverse ? head->offset : head->offset - delta;
-  preroll_start = MAX (preroll_start, 0);
-
-  preroll_end = head->reverse ? head->offset + delta : head->offset;
-  preroll_end = MIN (preroll_end, s->sounddata->nr_frames);
-
-  if (play_view (view, preroll_start, preroll_end,
-		 preroll_start, preroll_end, 1.0)) {
-    g_mutex_lock (head->head_mutex);
-    head->offset = head->stop_offset;
-    g_mutex_unlock (head->head_mutex);
-  }
-
-  device_drain (handle);
-
-  HEAD_SET_GOING (head, FALSE);
-
-  device_reset (handle);
-  device_close (handle);
-}
-#endif
-
 void
 play_preroll (sw_view * view)
-{
-#if 0
-  sw_handle * h;
-
-  sw_sample * s = view->sample;
-
-  stop_playback (prev_sample);
-
-  sample_set_playmode (s, SWEEP_TRANSPORT_PLAY);
-  sample_set_stop_offset (s);
-  sample_set_previewing (s, TRUE);
-
-  prev_sample = s;
-
-  if ((h = device_open(O_WRONLY)) != NULL) {
-    handle = h;
-#ifdef DEBUG
-    if (player_thread != -1) {
-      g_print ("Warning: runaway player thread!\n");
-    }
-#endif
-    pthread_create (&player_thread, NULL, (void *) (*pvpreroll), view);
-    start_playmarker (s);
-  } else {
-    sample_set_playmode (s, SWEEP_TRANSPORT_STOP);
-  }
-#endif
-}
-
-#ifdef DEVEL_CODE
-static void
-pvpreview_cut (sw_view * view)
 {
   sw_sample * s = view->sample;
   sw_head * head = s->play_head;
-  GList * gl, * gl_prev, * gl_next;
-  sw_sel * sel;
-  sw_framecount_t delta, preview_start, preview_end;
-  sw_framecount_t start = 0, end = 0;
-  gboolean was_reverse;
-  gboolean skip = FALSE; /* voodoo */
 
-  device_setup (handle, s->sounddata->format);
-
-  head_init_playback (s);
-
-  delta = time_to_frames (s->sounddata->format, 1.0);
-
-  g_mutex_lock (s->sounddata->sels_mutex);
-
-  sel = (sw_sel *)s->sounddata->sels->data;
-  preview_start = sel->sel_start;
-
-  for (gl = s->sounddata->sels; gl->next; gl = gl->next);
-  
-  sel = (sw_sel *)gl->data;
-  preview_end = sel->sel_end;
-  g_mutex_unlock (s->sounddata->sels_mutex);
-
-  gl = head->reverse ? gl : s->sounddata->sels;
-  
-  preview_start -= delta;
-  preview_start = MAX (preview_start, 0);
-
-  preview_end += delta;
-  preview_end = MIN (preview_end, s->sounddata->nr_frames);
-
-  g_mutex_lock (head->head_mutex);
-  head->offset = head->reverse ? preview_end : preview_start;
-  g_mutex_unlock (head->head_mutex);
-
-  while (gl) {
+  sample_set_stop_offset (s);
+  sample_set_previewing (s, TRUE);
+  sample_set_scrubbing (s, FALSE);
     
-    if (!head->going) break;
-    
-    /* Hold sels_mutex for as little time as possible */
-    g_mutex_lock (s->sounddata->sels_mutex);
-    
-    /* if gl is no longer in sels, break out */
-    if (g_list_position (s->sounddata->sels, gl) == -1) {
-      g_mutex_unlock (s->sounddata->sels_mutex);
-      break;
-    }
-    
-    sel = (sw_sel *)gl->data;
-    start = sel->sel_start;
-    end = sel->sel_end;
+  prev_sample = s;
 
-    gl_prev = gl->prev;
-    gl_next = gl->next;
+  head_set_restricted (head, FALSE);
 
-    g_mutex_unlock (s->sounddata->sels_mutex);
-
-    was_reverse = head->reverse;
-
-    if (!skip) {
-      if (head->reverse) {
-	play_view (view, end, head->offset, end, head->offset, 1.0);
-      } else {
-	play_view(view, head->offset, start, head->offset, start, 1.0);
-      }
-
-      skip = (was_reverse != head->reverse);
-    } else {
-      skip = FALSE;
-    }
-
-    g_mutex_lock (head->head_mutex);
-
-    if (head->reverse) {
-      head->offset = start;
-      gl = gl_prev;
-    } else {
-      head->offset = end;
-      gl = gl_next;
-    }
-
-    g_mutex_unlock (head->head_mutex);
-
-  }
-
-  if (head->going) {
-    if (head->reverse) {
-      play_view (view, preview_start, start, preview_start, start, 1.0);
-    } else {
-      play_view (view, end, preview_end, end, preview_end, 1.0);
-    }
-  }
-
-  g_mutex_lock (head->head_mutex);
-  head->offset = head->stop_offset;
-  g_mutex_unlock (head->head_mutex);
-  
-  if (head->going) {
-    device_drain (handle);
-
-    HEAD_SET_GOING (head, FALSE);
-  }
-
-  device_reset (handle);
-  device_close (handle);
+  sample_play (s);
 }
-#endif
 
 void
 play_preview_cut (sw_view * view)
 {
-#if 0
-  sw_handle * h;
-
   sw_sample * s = view->sample;
+  sw_head * head = s->play_head;
 
   if (s->sounddata->sels == NULL) {
-    sample_set_playmode (s, SWEEP_TRANSPORT_STOP);
+    head_set_going (head, FALSE);
+    sample_refresh_playmode (s);
     return;
   }
 
-  stop_playback (prev_sample);
-
-  sample_set_playmode (s, SWEEP_TRANSPORT_PLAY);
   sample_set_stop_offset (s);
   sample_set_previewing (s, TRUE);
-
+  sample_set_scrubbing (s, FALSE);
+    
   prev_sample = s;
 
-  if ((h = device_open(O_WRONLY)) != NULL) {
-    handle = h;
-#ifdef DEBUG
-    if (player_thread != -1) {
-      g_print ("Warning: runaway player thread\n");
-    }
-#endif
-    pthread_create (&player_thread, NULL, (void *) (*pvpreview_cut), view);
-    start_playmarker (s);
-  } else {
-    sample_set_playmode (s, SWEEP_TRANSPORT_STOP);
-  }
-#endif
-}
+  head_set_restricted (head, TRUE);
 
+  sample_play (s);
+}
 
 void
 play_view_all_pitch (sw_view * view, gfloat pitch)
@@ -1311,6 +1067,7 @@ play_view_all_pitch (sw_view * view, gfloat pitch)
   prev_sample = s;
 
   head_set_restricted (head, FALSE);
+  head_set_rate (head, pitch);
 
   sample_play (s);
 }
@@ -1319,8 +1076,8 @@ void
 stop_all_playback (void)
 {
   stop_all = TRUE;
-  g_list_free (active_play_heads);
-  active_play_heads = NULL;
+  g_list_free (active_main_heads);
+  active_main_heads = NULL;
 }
 
 void
@@ -1353,7 +1110,7 @@ stop_playback (sw_sample * s)
     sample_set_playmarker (s, head->stop_offset, TRUE);
 
     g_mutex_lock (play_mutex);
-    active_play_heads = g_list_remove (active_play_heads, head);
+    active_main_heads = g_list_remove (active_main_heads, head);
     g_mutex_unlock (play_mutex);
   }
 }
@@ -1361,5 +1118,11 @@ stop_playback (sw_sample * s)
 gboolean
 any_playing (void)
 {
-  return ((player_thread != -1) && (active_play_heads != NULL));
+  return ((player_thread != -1) && (active_main_heads != NULL));
+}
+
+void
+init_playback (void)
+{
+  play_mutex = g_mutex_new ();
 }
