@@ -1321,7 +1321,7 @@ sd_pulse_cursor (gpointer data)
 
   s->pulse = (!s->pulse);
 
-  sample_display_refresh_play_marker (s);
+  sample_display_refresh_user_marker (s);
 
   return TRUE;
 }
@@ -1331,7 +1331,7 @@ sample_display_start_cursor_pulse (SampleDisplay * s)
 {
   gdk_gc_set_function (s->user_gc, GDK_NOOP);
 
-  sample_display_refresh_play_marker (s);
+  sample_display_refresh_user_marker (s);
 
   s->pulsing_tag = gtk_timeout_add (PULSE_INTERVAL,
 				    (GtkFunction)sd_pulse_cursor, s);
@@ -1347,7 +1347,7 @@ sample_display_stop_cursor_pulse (SampleDisplay * s)
 
   gdk_gc_set_function (s->user_gc, GDK_COPY);
 
-  sample_display_refresh_play_marker (s);
+  sample_display_refresh_user_marker (s);
 }
 
 static void
@@ -1740,8 +1740,9 @@ sample_display_handle_sel_motion (SampleDisplay *s,
   }
 #endif
 
-  if (!sample->play_head->going)
+  if (!sample->play_head->going) {
     sample_set_playmarker (sample, o, TRUE);
+  }
 
   sel = sample->tmp_sel;
 
@@ -2133,6 +2134,7 @@ sample_display_button_press (GtkWidget      *widget,
 	/* If the cursor is near a sel, move that */
 	if (sample_display_on_sel (s, x, y)) {
 	  sample_display_handle_sel_button_press (s, x, y, state);
+#if 0
 	} else if (sample_display_on_playmarker (s, x, y)) {
 	  /* If the cursor is near the play marker, move that */
 	  s->selecting = SELECTING_PLAYMARKER;
@@ -2140,6 +2142,7 @@ sample_display_button_press (GtkWidget      *widget,
 #ifndef SEL_SCRUBS
 	  sample_set_scrubbing (sample, TRUE);
 	  sample_display_handle_playmarker_motion (s, x, y);
+#endif
 #endif
 	} else {
 	  sample_display_handle_sel_button_press (s, x, y, state);
@@ -2152,9 +2155,9 @@ sample_display_button_press (GtkWidget      *widget,
 	 * upon release. */
 	if (sample->play_head->going) {
 	  sample_set_scrubbing (sample, FALSE);
-	  head_set_restricted (sample->play_head, FALSE);
 	  sample_refresh_playmode (sample);
 	} else {
+	  /*sample_set_monitor (sample, TRUE);*/
 	  sample_set_scrubbing (sample, TRUE);
 	  play_view_all (s->view);
 	  sample_display_handle_playmarker_motion (s, x, y);
@@ -2569,6 +2572,10 @@ sample_display_key_press (GtkWidget * widget, GdkEventKey * event)
   case GDK_KP_Left:
     move_delta = MIN(-1,  -vlen/s->width);
     if (event->state & GDK_CONTROL_MASK) {
+      if (!(event->state & GDK_SHIFT_MASK)) {
+	sample_set_offset_next_bound_left (sample);
+	return TRUE;
+      }
       move_delta *= 10;
     }
     break;
@@ -2576,6 +2583,10 @@ sample_display_key_press (GtkWidget * widget, GdkEventKey * event)
   case GDK_KP_Right:
     move_delta = MAX(1,  vlen/s->width);
     if (event->state & GDK_CONTROL_MASK) {
+      if (!(event->state & GDK_SHIFT_MASK)) {
+	sample_set_offset_next_bound_right (sample);
+	return TRUE;
+      }
       move_delta *= 10;
     }
     break;
@@ -2584,6 +2595,8 @@ sample_display_key_press (GtkWidget * widget, GdkEventKey * event)
     break;
   }
 
+  /* Handle movement only from here on */
+
   if ((event->state & GDK_SHIFT_MASK) &&
       (sample->edit_mode == SWEEP_EDIT_MODE_READY)) {
     sel = sample->tmp_sel;
