@@ -180,7 +180,6 @@ sample_new_empty(char * pathname,
   s->color = filename_color_hash (g_basename (s->pathname));
 #endif
 
-  s->paste_dialog = NULL;
   s->info_clist = NULL;
 
   /*  s->recording = FALSE;*/
@@ -379,7 +378,7 @@ sample_new_dialog_defaults_cb (GtkWidget * widget, gpointer data)
 }
 
 static void
-create_sample_new_dialog (gint nr_channels, gint sample_rate,
+create_sample_new_dialog (gchar * pathname, gint nr_channels, gint sample_rate,
 			  sw_time_t duration, gboolean do_reset)
 {
   GtkWidget * dialog;
@@ -448,7 +447,8 @@ create_sample_new_dialog (gint nr_channels, gint sample_rate,
   gtk_box_pack_start (GTK_BOX(hbox), entry, TRUE, TRUE, 4);
   gtk_widget_show (entry);
 
-  gtk_entry_set_text (GTK_ENTRY (entry), filename_generate ()); 
+  gtk_entry_set_text (GTK_ENTRY (entry), 
+		      pathname ? pathname : filename_generate ()); 
 
   gtk_object_set_data (GTK_OBJECT (dialog), "name_entry", entry);
 
@@ -633,9 +633,10 @@ create_sample_new_dialog (gint nr_channels, gint sample_rate,
 }
 
 void
-create_sample_new_dialog_defaults (void)
+create_sample_new_dialog_defaults (char * pathname)
 {
-  create_sample_new_dialog (DEFAULT_CHANNELS, DEFAULT_SAMPLERATE, 60, TRUE);
+  create_sample_new_dialog (pathname, DEFAULT_CHANNELS, DEFAULT_SAMPLERATE,
+			    60, TRUE);
 }
 
 void
@@ -643,7 +644,7 @@ create_sample_new_dialog_like (sw_sample * s)
 {
   sw_format * f = s->sounddata->format;
 
-  create_sample_new_dialog (f->channels, f->rate,
+  create_sample_new_dialog (NULL, f->channels, f->rate,
 			    frames_to_time (f, s->sounddata->nr_frames),
 			    FALSE);
 }
@@ -966,7 +967,8 @@ sample_set_playmarker (sw_sample * s, sw_framecount_t offset,
   sw_head * head = s->play_head;
 
 #ifdef DEBUG
-  g_print ("sample_set_playmarker ()\n");
+  g_print ("sample_set_playmarker (%p, %d, %s)\n", s, offset,
+	   by_user ? "TRUE" : "FALSE");
 #endif
 
   g_mutex_lock (s->play_mutex);
@@ -996,6 +998,44 @@ sample_set_playmarker (sw_sample * s, sw_framecount_t offset,
   }
 }
 
+void
+sample_set_offset_next_bound_left (sw_sample * s)
+{
+  GList * gl;
+  sw_sel * sel;
+
+  for (gl = g_list_last (s->sounddata->sels); gl; gl = gl->prev) {
+    sel = (sw_sel *)gl->data;
+    if (sel->sel_end < s->user_offset) {
+      sample_set_playmarker (s, sel->sel_end, TRUE);
+      return;
+    }
+    if (sel->sel_start < s->user_offset) {
+      sample_set_playmarker (s, sel->sel_start, TRUE);
+      return;
+    }
+  }
+}
+
+void
+sample_set_offset_next_bound_right (sw_sample * s)
+{
+  GList * gl;
+  sw_sel * sel;
+
+  for (gl = s->sounddata->sels; gl; gl = gl->next) {
+    sel = (sw_sel *)gl->data;
+    if (sel->sel_start > s->user_offset) {
+      sample_set_playmarker (s, sel->sel_start, TRUE);
+      return;
+    }
+    if (sel->sel_end > s->user_offset) {
+      sample_set_playmarker (s, sel->sel_end, TRUE);
+      return;
+    }
+  }
+}
+
 void
 sample_set_rec_marker (sw_sample * s, sw_framecount_t offset)
 {
@@ -1120,6 +1160,21 @@ sample_set_mute (sw_sample * s, gboolean mute)
 #endif
 }
 
+void
+sample_set_monitor (sw_sample * s, gboolean monitor)
+{
+  GList * gl;
+  sw_view * v;
+  sw_head * head = s->play_head;
+
+  head_set_monitor (head, monitor);
+
+  for(gl = s->views; gl; gl = gl->next) {
+    v = (sw_view *)gl->data;
+    view_refresh_monitor (v);
+  }
+}
+
 void
 sample_set_previewing (sw_sample * s, gboolean previewing)
 {
@@ -1624,12 +1679,15 @@ sample_selection_insert_tmp_sel (sw_sample * s)
 #undef BUF_LEN
 #define BUF_LEN 64
   gchar buf[BUF_LEN];
+  sw_format * format = s->sounddata->format;
   sw_sel * sel;
 
   n = snprintf (buf, BUF_LEN, "Insert selection [");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_start);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_start));
   n += snprintf (buf+n, BUF_LEN-n, " - ");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_end);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_end));
   n += snprintf (buf+n, BUF_LEN-n, "]");
 
   g_mutex_lock (s->sounddata->sels_mutex);
@@ -1674,12 +1732,15 @@ sample_selection_subtract_tmp_sel (sw_sample * s)
   int n;
 #define BUF_LEN 64
   gchar buf[BUF_LEN];
+  sw_format * format = s->sounddata->format;
   sw_sel * sel;
 
   n = snprintf (buf, BUF_LEN, "Subtract selection [");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_start);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_start));
   n += snprintf (buf+n, BUF_LEN-n, " - ");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_end);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_end));
   n += snprintf (buf+n, BUF_LEN-n, "]");
 
   g_mutex_lock (s->sounddata->sels_mutex);
@@ -1716,12 +1777,15 @@ sample_selection_replace_with_tmp_sel (sw_sample * s)
   int n;
 #define BUF_LEN 64
   gchar buf[BUF_LEN];
+  sw_format * format = s->sounddata->format;
   sw_sel * sel;
 
   n = snprintf (buf, BUF_LEN, "Set selection [");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_start);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_start));
   n += snprintf (buf+n, BUF_LEN-n, " - ");
-  n += snprint_time (buf+n, BUF_LEN-n, s->tmp_sel->sel_end);
+  n += snprint_time (buf+n, BUF_LEN-n,
+		     frames_to_time (format, s->tmp_sel->sel_end));
   n += snprintf (buf+n, BUF_LEN-n, "]");
 
   g_mutex_lock (s->sounddata->sels_mutex);
