@@ -22,11 +22,17 @@
 #include <string.h>
 #include <glib.h>
 
+#if 0
+#include <unistd.h>
+#include <sys/mman.h>
+#endif
+
 #include <sweep/sweep_types.h>
 #include <sweep/sweep_typeconvert.h>
 #include <sweep/sweep_selection.h>
 #include <sweep/sweep_undo.h>
 
+#include "edit.h"
 #include "format.h"
 #include "view.h"
 #include "sample-display.h"
@@ -50,7 +56,14 @@ sounddata_new_empty(gint nr_channels, gint sample_rate, gint sample_length)
 
   if (sample_length > 0) {
     len = frames_to_bytes (s->format, sample_length);
+
+#if 1
     s->data = g_malloc0 ((size_t)len);
+#else
+    s->data = sweep_large_alloc_zero (len, PROT_READ|PROT_WRITE);
+#endif
+
+
     if (!(s->data)) {
 #if 0
 #if (SIZEOF_OFF_T == 8)
@@ -89,10 +102,17 @@ sounddata_clear_selection (sw_sounddata * sounddata)
 void
 sounddata_destroy (sw_sounddata * sounddata)
 {
+  /*size_t len;*/
+
   sounddata->refcount--;
 
   if (sounddata->refcount <= 0) {
+#if 1
     g_free (sounddata->data);
+#else
+    len = frames_to_bytes (sounddata->format, sounddata->nr_frames);
+    munmap (sounddata->data, len);
+#endif
     sounddata_clear_selection (sounddata);
     g_free (sounddata);
   }
