@@ -786,7 +786,6 @@ paste_over_data_destroy (paste_over_data * p)
   edit_buffer_destroy (p->old_eb);
   if (p->new_eb != p->old_eb)
     edit_buffer_destroy (p->new_eb);
-
   g_free (p);
 }
 
@@ -826,12 +825,7 @@ splice_data_destroy (splice_data * s)
 void
 undo_by_splice_in (sw_sample * s, splice_data * sp)
 {
-  sw_sounddata * out;
-
-  out = splice_in_eb (s->sounddata, sp->eb);
-
-  sounddata_destroy (s->sounddata);
-  s->sounddata = out;
+  splice_in_eb (s, sp->eb);
 }
 
 void
@@ -851,12 +845,21 @@ undo_by_splice_out (sw_sample * s, splice_data * sp)
 void
 redo_by_splice_in (sw_sample * s, splice_data * sp)
 {
-  sw_sounddata * out;
+  splice_in_eb (s, sp->eb);
+}
 
-  out = splice_in_eb (s->sounddata, sp->eb);
+void
+undo_by_splice_over (sw_sample * s, splice_data * sp)
+{
+  s->sounddata->sels = sels_copy (sp->sels);
+  paste_over (s, sp->eb);
+}
 
-  sounddata_destroy (s->sounddata);
-  s->sounddata = out;
+void
+redo_by_splice_over (sw_sample * s, splice_data * sp)
+{
+  s->sounddata->sels = sels_copy (sp->sels);
+  paste_over (s, sp->eb);
 }
 
 void
